#!/bin/bash

source .env

TASKS_DIR="tasks"

# dns entry for docker image registry
ip_address="127.0.0.1"
hostname=$DOCKER_REGISTRY_HOSTNAME

# check if entry exists
if ! grep -q "$ip_address\s*$hostname" /etc/hosts; then

    read -p "This script will add the DNS entry '$ip_address $hostname' to /etc/hosts. Do you want to proceed? [Y/n] " choice
    choice="${choice:-y}"

    if [[ $choice =~ ^[Yy]$ ]]; then
        echo -e "\n# for eva demo\n$ip_address $hostname" | sudo tee -a /etc/hosts >/dev/null
        echo "Added dns entry '$ip_address $hostname'"
    fi
fi

if [ ! -f "htpasswd" ]; then
    # prepare auth for docker image registry
    echo "Generating htpasswd"
    docker run --entrypoint htpasswd httpd:2 -Bbn $DOCKER_REGISTRY_USER $DOCKER_REGISTRY_PASSWORD > htpasswd

    echo "Logging into docker image registry"
    docker login docker.eva.local:5000 -u $DOCKER_REGISTRY_USER -p $DOCKER_REGISTRY_PASSWORD
fi

# get some example tasks
if [ ! -d "$TASKS_DIR" ]; then
    mkdir $TASKS_DIR && git clone git@gitlab.cs.hs-rm.de:sls/stef-examples.git $TASKS_DIR/stef-examples
fi

# build some environments that will be available for evaluation
../environments/build.sh $ENVIRONMENTS