#!/usr/bin/env bash

set -e

if [ "$#" -lt 1 ]; then
    echo "Usage: $0 dir1 [dir2 [dir3] ...]"
    exit 1
fi

PREFIX=eva/environments

build() {
    local path=$1
    local env=$(basename $path)
    local image="$PREFIX/$env:latest"

    if [ -d "$path" ]; then
        echo "Building docker image for $env"
        (cd $path && docker build -t $image -f Dockerfile ../../) # use --no-cache if apt sources outdated

        echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        echo "Image: $image ($path)"
        echo "You can now use this environment LOCALLY."
        echo "CHECKLIST"
        echo "[ ] all files checked in via git."
        echo "[ ] debug outputs are commented out"
        echo "[ ] integration tests have been created in eva/pltform/tests/integration"
        echo "[ ] files that are shared by multiple environments are not copy-pasted if they are identical. Please use the environments/shared folder."
        echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    else
        echo "Directory $path does not exist"
    fi
}

for ENVIRON in "$@"; do
    build $ENVIRON
done

echo "removing old dangling images"
docker image prune -f