#include "ads.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>


IntArray mkIntArray(size_t n) {
    IntArray rs;
    rs.length = n;

    rs.data = (int*) malloc(n * sizeof(int));
    if (rs.data == NULL) exit(EXIT_FAILURE);

    for (int i = 0; i < rs.length; i++) {
        rs.data[i] = 0;
    }

    return rs;
}

void delIntArray(IntArray dataArray) {
    free(dataArray.data);
}

BoolArray mkBoolArray(size_t n) {
    BoolArray rs;
    rs.length = n;

    rs.data = (bool*) malloc(n * sizeof(bool));
    if (rs.data == NULL) exit(EXIT_FAILURE);

    for (int i = 0; i < rs.length; i++) {
        rs.data[i] = false;
    }

    return rs;
}

BoolArray copyBoolArray(BoolArray dataArray) {
    BoolArray rs = mkBoolArray(dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        rs.data[i] = dataArray.data[i];
    }

    return rs;
}

void printBoolArray(BoolArray dataArray) {
    printf("Bool[len=%ld; ", dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        if (i == dataArray.length-1) {
            printf("%d", dataArray.data[i]);
        } else {
            printf("%d, ", dataArray.data[i]);
        }
    }
    fflush(stdout);
    printf("]\n");
}

void delBoolArray(BoolArray dataArray) {
    free(dataArray.data);
}

void swap(IntArray dataArray, int pos1, int pos2) {
    int tmp = dataArray.data[pos1];
    dataArray.data[pos1] = dataArray.data[pos2];
    dataArray.data[pos2] = tmp;
}

void printArray(IntArray dataArray) {
    printf("Int[");
    for (int i = 0; i < dataArray.length; i++) {
        if (i == dataArray.length-1) {
            printf("%d", dataArray.data[i]);
        } else {
            printf("%d, ", dataArray.data[i]);
        }
    }
    fflush(stdout);
    printf("]\n");
}

void shuffleArray(int seed, IntArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = i;
    }

    // immer dieselben Pseudozufallszahlen
    srand(seed * dataArray.length);

    for (int j = 0; j < dataArray.length; j++) {
        int pos = rand() % dataArray.length;
        swap(dataArray, j, pos);
    }
}

IntArray copyArray(IntArray dataArray) {
    IntArray rs = mkIntArray(dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        rs.data[i] = dataArray.data[i];
    }

    return rs;
}

void fillArrayByData(IntArray dataArray, int data[]) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = data[i];
    }
}

void fillArrayByIndex(IntArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = i;
    }
}

void fillArrayWithZero(IntArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = 0;
    }
}

// functions DoubleArray
DoubleArray mkDoubleArray(size_t n) {
    DoubleArray rs;
    rs.length = n;

    rs.data = (double*) malloc(n * sizeof(double));
    if (rs.data == NULL) exit(EXIT_FAILURE);

    for (int i = 0; i < rs.length; i++) {
        rs.data[i] = 0;
    }

    return rs;
}

DoubleArray copyDoubleArray(DoubleArray dataArray) {
    DoubleArray rs = mkDoubleArray(dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        rs.data[i] = dataArray.data[i];
    }

    return rs;
}

void fillDoubleArrayByData(DoubleArray dataArray, double data[]) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = data[i];
    }
}

void fillDoubleArrayWithZero(DoubleArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = 0;
    }
}

void delDoubleArray(DoubleArray dataArray) {
    if(dataArray.data)
     free(dataArray.data);
}

void printDoubleArray(DoubleArray dataArray) {
    printf("Double[");
    for (int i = 0; i < dataArray.length; i++) {
        if (i == dataArray.length-1) {
            printf("%lf", dataArray.data[i]);
        } else {
            printf("%lf, ", dataArray.data[i]);
        }
    }
    fflush(stdout);
    printf("]\n");
}

//Short
// functions ShortArray
void initShortArray(ShortArray *rs, size_t n) {
    rs->length = n;

    rs->data = (short*) malloc(n * sizeof(short));
    if (rs->data == NULL) exit(EXIT_FAILURE);

    for (int i = 0; i < rs->length; i++) {
        rs->data[i] = 0;
    }
}

ShortArray mkShortArray(size_t n) {
    ShortArray rs;
    initShortArray(&rs, n);
    return rs;
}

ShortArray copyShortArray(ShortArray dataArray) {
    ShortArray rs = mkShortArray(dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        rs.data[i] = dataArray.data[i];
    }

    return rs;
}

void fillShortArrayByData(ShortArray dataArray, short data[]) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = data[i];
    }
}

void fillShortArrayWithZero(ShortArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = 0;
    }
}

void delShortArray(ShortArray dataArray) {
    if(dataArray.data)
     free(dataArray.data);
}

void printShortArray(ShortArray dataArray) {
    printf("Short[");
    for (int i = 0; i < dataArray.length; i++) {
        if (i == dataArray.length-1) {
            printf("%u", dataArray.data[i]);
        } else {
            printf("%u, ", dataArray.data[i]);
        }
    }
    fflush(stdout);
    printf("]\n");
}


//Char
// functions CharArray
void initCharArray(CharArray *rs, size_t n) {


    rs->length = n;

    if(n==0) {
        rs->data = 0;
        return;
    }

    rs->data = (char*) malloc(n * sizeof(char));
    if (rs->data == NULL) exit(EXIT_FAILURE);

    for (int i = 0; i < rs->length; i++) {
        rs->data[i] = 0;
    }
}

CharArray mkCharArray(size_t n) {
    CharArray rs;
    initCharArray(&rs, n);
    return rs;
}

bool equalsCharArray(CharArray a, CharArray b) {
	if(a.length != b.length) return false;

	for(int i=0;i<a.length;++i) {
		if(a.data[i] != b.data[i]) return false;
	}
	return true;
}

void copyCharArray(CharArray *src, CharArray *dest) {
    if(src->length != dest->length) {
        printf("ERROR: Can't copy. CharArray lengths do not match!");
        return;
    }
    memcpy(dest->data, src->data, sizeof(char) * dest->length);
}

CharArray duplicateCharArray(CharArray dataArray) {
    CharArray rs = mkCharArray(dataArray.length);
    for (int i = 0; i < dataArray.length; i++) {
        rs.data[i] = dataArray.data[i];
    }

    return rs;
}

void fillCharArrayByData(CharArray dataArray, char data[]) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = data[i];
    }
}

void fillCharArrayWithZero(CharArray dataArray) {
    for (int i = 0; i < dataArray.length; i++) {
        dataArray.data[i] = 0;
    }
}

void delCharArray(CharArray dataArray) {
    if(dataArray.data && dataArray.length>0) {
        free(dataArray.data);
        dataArray.length = 0;
    }
    dataArray.length = 0;
}

void printCharArray(CharArray dataArray) {
    printf("Char[");
    for (int i = 0; i < dataArray.length; i++) {
        if (i == dataArray.length-1) {
            printf("%d", dataArray.data[i]);
        } else {
            printf("%d, ", dataArray.data[i]);
        }
    }
    fflush(stdout);
    printf("]\n");
}

CharArray charArrayFromInitalizerlist(int count, ...) {
	va_list args;
    va_start(args, count);

	CharArray m = mkCharArray(count);

	for (int i = 0; i < count; ++i) {
        m.data[i] = va_arg(args, int);
    }

	va_end(args);
	return m;
	}




