



> module Evarela where
> import Data.Char
> import Control.Monad
> import Control.Exception
> import System.Console.Readline
> import Data.List(isPrefixOf,transpose,partition,genericLength)
> import OurParserLib





> data Type = TInt|TString|TReal  deriving Eq





> instance Show Type where
>   show TInt = "int"
>   show TString = "string"
>   show TReal = "real"





> data Value  = I Integer|S String|D Double deriving Eq





> instance Show Value where
>  show (I i) = show i
>  show (S s) = show s
>  show (D d) = show d





> instance Ord Value where
>  (I i1) <= (I i2) = i1<=i2
>  (D i1) <= (D i2) = i1<=i2
>  (S i1) <= (S i2) = i1<=i2
>  (I _) <= _ = True
>  (D _) <= (S d) = True
>  _ <= _ = False





> type Row = [Value]
> type Table = [Row]

> type Ident = String
> type Relation = (Ident,[(Ident,Type)])
> type Rel = ([(Ident,Type)],Table)





> data Expression
>   =  Relation Ident
>     |Union Expression Expression
>     |Join (Maybe Expr) Expression Expression
>     |Projection Expression [Ident]
>     |Selection Expression Expr
>     |Rename Ident Ident Expression
>     |Fun ColumnFun  Expression
>     |GroupOP [Attribute] [(ColumnFun,Attribute)]  Expression

> type ColumnFun = [Value] -> Value





> data Expr =
>    Expr :/\ Expr -- and
>  | Expr :\/ Expr -- or
>  | Not Expr
>  | Lt Attribute ValOrAttr
>  | Gt Attribute ValOrAttr
>  | Le Attribute ValOrAttr
>  | Ge Attribute ValOrAttr
>  | Eq Attribute ValOrAttr
>  | Ne Attribute ValOrAttr
>     deriving Show





> type Attribute = String
> data ValOrAttr = V Value| A Attribute deriving Show





> type ErrMsg = String

> eval :: [(Ident,Rel)]  -> [(Ident,Expression)] -> Expression -> Either ErrMsg Rel





> eval rels defs (Relation n) = maybe (storedQuery n) (\x->Right x) rel
>   where
>     rel = lookup n rels
>     storedQuery n = maybe (Left ("Unbekannter Bezeichner: "++n)) (eval rels defs) mq
>       where
>         mq = lookup n defs





> eval rels defs (Rename old new e) = case r1 of
>   Left _ -> r1
>   Right (ts,rs) -> Right (map (\(n,t)-> if n==old then (new,t)else (n,t)) ts,rs)
>  where
>   r1 = eval rels defs e





> eval rels defs (Projection e n) = case r1 of
>   Left _ -> r1
>   Right (ts,rs) -> projection ts rs n
>  where
>   r1 = eval rels defs e





>   projection ts rs as
>    |length nums > length as
>        = Left ("falsche Attribute "++show as++" in "++show ts)
>    |otherwise = Right (getnums ts,map getnums rs)
>     where
>       nums       = concat$ map  (\a->[n |((t,_),n)<-zip ts [0,1..], a == t ]) as
>       getnums xs = map (\n->xs !! n) nums





> eval rels defs (Selection r e) = case r1 of
>   Left _ -> r1
>   Right (ts,rs) -> selection ts rs e
>  where
>   r1 = eval rels defs r





> eval rels defs (Union e1 e2) = rights union r1 r2
>  where
>   r1 = eval rels defs e1
>   r2 = eval rels defs e2
>   union (typ1,t1s) (typ2,t2s)
>     |typ1 /= typ2
>       = Left ("Vereinigung unterschiedlicher Tabellentypen: "
>                ++show typ1++" und "++ show typ2)
>     |otherwise    = Right (typ1,t1s++t2s)





> eval rels defs (Join Nothing e1 e2) = rights (|><|) r1 r2
>  where
>   r1 = eval rels defs e1
>   r2 = eval rels defs e2
>   (typ1,t1s) |><| (typ2,t2s)
>     |null common = Left ("Join nicht möglich. Keine gemeinsamen Spaltennamen.")
>     |otherwise   = Right (typ1++getnotnums num2 typ2,rs)
>     where
>      common = [fst t |t<-typ1 , elem t typ2]
>      num1   = [n |((t,_),n)<-zip typ1 [0,1..], elem t common ]
>      num2   = [n |((t,_),n)<-zip typ2 [0,1..], elem t common ]
>      getnums nums xs = [x |(x,n)<-zip xs [0,1..],elem n nums]
>      getnotnums nums xs = [x |(x,n)<-zip xs [0,1..],not$elem n nums]
>      rs     = [row1++getnotnums num2 row2
>               |row1<-t1s
>               ,row2<-[row |row<-t2s,getnums num1 row1==getnums num2 row ]
>               ]





> eval rels defs (Join (Just p) e1 e2) = rights (joinP p) r1 r2
>  where
>   r1 = eval rels defs e1
>   r2 = eval rels defs e2
>   joinP p (typ1,t1s) (typ2,t2s) =
>      Right (ts,[rs
>                | r1<-t1s
>                , r2<-t2s
>                , let rs = r1++r2
>                , select (zip (map fst ts) [0,1..]) rs p])
>    where
>      ts = typ1++typ2





> eval rels defs (Fun f e) = case r1 of
>   Left _ -> r1
>   Right erg -> apply erg
>  where
>   r1 = eval rels defs e
>   apply (ts,rss)
>    |length ts==1 && not (null rs) = Right (ts,[[f rs]])
>    |otherwise = Left ("Funktion auf Relation "++show ts++" nicht anwendbar")
>    where rs = concat rss





> eval rels defs (GroupOP as fas  e) = case r1 of
>   Left _ -> r1
>   Right erg -> group erg
>  where
>   r1 = eval rels defs e
>   group (ts,rss) = Right (project nas ts++take (length fas) [("R"++show n,TInt)|n<-[1,2..]] , result)
>     where
>       nas :: [Int]
>       nas = nums as ts

>       gps :: [Table]
>       gps = equivalenceClasses (\r1 r2 -> project nas r1==project nas r2) rss

>       agregate (f,col) = f col

>       agregates :: Table -> [Value]
>       agregates gss =
>          map agregate$zip (map fst fas) columns
>         where
>            columns :: [[Value]]
>            columns =  getnums (nums (map snd fas) ts) gss


>       result = map (\gr-> (project nas (head gr)++agregates gr)) gps





> splitOffFirstGroup :: (a -> a -> Bool) -> [a] -> ([a],[a])
> splitOffFirstGroup equal xs@(x:_) = partition (equal x) xs
> splitOffFirstGroup _     []       = ([],[])

> equivalenceClasses _     [] = []
> equivalenceClasses equal xs = let (fg,rst) = splitOffFirstGroup equal xs
>                               in fg : equivalenceClasses equal rst

> nat = [0::Int,1..]
> nums as ts   --   = [n |((t,_),n)<-zip ts nat, elem t as ]
>   = concat$ map  (\a->[n |((t,_),n)<-zip ts [0,1..], a == t ]) as

> getnums is xss = transpose$map (project is) xss

> project :: [Int] -> [b] -> [b]
> project is xs = map (\i->xs!!i) is
>    -- [x |(x,n)<-zip xs nat,elem n is]





> selection ts rs e = Right (ts,[r|r<-rs,select (zip (map fst ts) [0,1..]) r e ])

> select ts row (e1 :/\ e2) = select ts row e1 && select ts row e2
> select ts row (e1 :\/ e2) = select ts row e1 || select ts row e2
> select ts row (Not e) = not$select ts row e
> select ts row (Eq attr val) = comparemaybe ts row (==) val$selectAttr ts attr row
> select ts row (Le attr val) = comparemaybe  ts row (<=) val$selectAttr ts attr row
> select ts row (Ge attr val) = comparemaybe ts row (>=) val$selectAttr ts attr row
> select ts row (Lt attr val) = comparemaybe ts row (<) val$selectAttr ts attr row
> select ts row (Gt attr val) = comparemaybe ts row (>) val$selectAttr ts attr row
> select ts row (Ne attr val) = select ts row (Not (Eq attr val))

> selectAttr ts attr row = fmap (\x->row!!x) $lookup attr ts

> comparemaybe ts row op (V v) vm = maybe False (\x -> op x v) vm
> comparemaybe ts row op (A a) vm = maybe False (\x -> maybe False (\v -> op x v) attrm) vm
>   where
>      attrm = selectAttr ts a row





> rights _ (Left e1)(Left e2) = Left (e1++"\n"++e2)
> rights _ (Left e1) _ = Left e1
> rights _ _ (Left e2)  = Left e2
> rights op (Right t1) (Right t2) = op t1 t2





> tInt = keyword "int" <<- \_ ->TInt
> tString = keyword "string" <<- \_ ->TString
> tReal = keyword "real" <<- \_ ->TReal

> pType = tInt !|+ tString !|+ tReal





> pName :: Parser Char Ident
> pName =    (satisfy (\x-> isAlpha x || elem x "_"))
>         +- (zeroToN
>             $satisfy (\x ->(isAlpha x||isDigit x||elem x "_") && (not$isSpace x)))
>          <<- (\(x,xs)->x:xs)





> pColDef = pName -+- terminal ':' -+- pType <<- \((n,_),typ)->(n,typ)

> pRelDef :: Parser Char Relation
> pRelDef =
>         pName
>     -+- terminal '('
>     -+- sepByOne (ignoreSpace pColDef) (ignoreSpace(terminal ','))
>     -+- terminal ')' <<- \((((n,_),defs),_)) -> (n,defs)





> pRelDefs = oneToN pRelDef

> pRelDataDef =
>   keyword "def" -+- keyword "table" -+-
>   (pRelDef -+- pTable <<- \((name,ts),rs)->(name,(ts,rs)))
>    <<- snd

> pRelDataDefs = oneToN (ignoreSpace pRelDataDef)

> bsp1 = pRelDef "R(A : int ,  B:int)"





> pDouble =
>  pInt +- terminal '.' +- pInt
>    <<- \((i1,_),i2) -> fromInteger i1*1.0 + toDec (fromInteger i2)

> toDec d
>      |d>=1.0 = toDec (d/10)
>      |otherwise = d


> pInt = oneToN (satisfy isDigit) <<- \xs ->  (read xs)::Integer

> pString = terminal '\"' +- zeroToN (satisfy (/= '\"')) +- terminal '\"'
>                  <<- \((_,str),_) -> str

> pValue = pDouble <<- D !|+ pInt <<- I !|+ pString <<- S

> pRow = sepByOne (ignoreInlineSpace pValue) (terminal ' ')

> ignoreInlineSpace p = \xs -> p (dropWhile (\x->isSpace x && not (x=='\n')) xs)

> pTable = sepByOne (ignoreSpace pRow) (terminal '\n')





> pQueryDef = pName -+- keyword ":=" -+- pQuery <<- (\((n,_),e)-> (n,e))





> pQuery = pGamma !|+ pFunCall !|+ pQuery2





> pGamma = keyword "GAMMA"
>          -+- pAtrs
>          -+- (terminal '[' -+- pFunPros -+- terminal ']'  <<- middle)
>          -+- (terminal '(' -+- pQuery2 -+- terminal ')' <<- middle)
>            <<- \(((_,atrs),funs),e) -> GroupOP atrs funs e

> pFunPros =  sepByOne (ignoreSpace pFunPro)  (ignoreSpace$terminal ',')

> pFunPro = pFun -+- (terminal '(' -+- pName -+- terminal ')' <<- middle)

> middle = snd.fst





> pFunCall = pFun -+- pQuery2 <<- \(f,e)-> Fun f e





> pFun =
>       (keyword "MAX"  <<- \_ -> (maximum::ColumnFun))
>   !|+ (keyword "MIN"  <<- \_ -> (minimum::ColumnFun))
>   !|+ (keyword "SUM"  <<- \_ -> (foldr1 sumVal))
>   !|+ (keyword "SIZE" <<- \_ -> I . genericLength)





> sumVal (I i1)(I i2) = I (i1+i2)
> sumVal (D i1)(D i2) = D (i1+i2)
> sumVal (S i1)(S i2) = S (i1++i2)
> sumVal _ _ = error ""





> pQuery2:: Parser  Char Expression
> pQuery2 = pProjection -+- (zeroToN (ignoreSpace pBinOp))
>                          <<- \(r,fs)-> foldl (\r f -> f r) r fs





> pBinOp:: Parser Char (Expression -> Expression)
> pBinOp = pUnion !|+ pJoin





> pUnion = keyword "UNION"  -+- pProjection <<- \(_,e2)-> \e1 -> Union e1 e2





> pJoin =       keyword "|><|"
>          -+- (zeroOrOne (terminal '['-+-pExpr-+-terminal ']' <<- \((_,e),_)->e))
>          -+- pProjection
>             <<- \((_,p),e2) ->  \e1 -> (Join p e1 e2)





> pProjection:: Parser  Char Expression
> pProjection =
>   pAtom -+- (zeroToN pQualifier)
>    <<- \(n,mpros) -> foldl (\r pro -> pro r) n mpros





> pQualifier=
>          pSingleProj
>      !|+ pMultProj
>      !|+ pPredicate
>      !|+ pRename





> pSingleProj = terminal '.' -+- pName        <<- (\x -> \e-> Projection e [snd x])





> pAtrs :: Parser Char [Attribute]
> pAtrs =          terminal '['
>              -+- sepByOne (ignoreSpace pName) (ignoreSpace$terminal ',')
>              -+- terminal ']' <<- middle

> pMultProj = pAtrs <<- \as -> (\e-> Projection e as)





> pPredicate =  ((terminal '[' -+- pExpr -+- terminal ']')
>                                         <<- (\((_,e),_)-> \e2 -> Selection e2 e))





> pRename =  ((terminal '[' -+- pName -+- keyword "->" -+- pName -+- terminal ']' )
>               <<- (\((((_,old),_),new),_)-> \e2 -> Rename old new e2))





> pAtom =      pName <<- Relation
>         !|+  terminal '(' -+- pQuery -+- terminal ')' <<- middle





> pSkript = zeroToN (ignoreSpace pRelDef) +-  zeroToN(ignoreSpace pQueryDef)






> pExpr   = pOrExpr  +- (zeroToN (ignoreSpace (keyword "/\\" -+- pExpr   <<- snd)))
>                <<- (\(x,xs)->foldl (\r y-> r :/\ y) x xs)

> pOrExpr :: Parser Char Expr
> pOrExpr = pNotExpr +- (zeroToN (ignoreSpace (keyword "\\/" -+- pOrExpr <<- snd)))
>                <<- (\(x,xs)->foldl (\r y-> r:\/ y) x xs)

> pNotExpr :: Parser Char Expr
> pNotExpr =     keyword "not" -+- pCompareExpr <<- Not . snd
>            !|+ pCompareExpr


> pCompareExpr :: Parser Char Expr
> pCompareExpr = pAttribute -+- pCmpOp -+- (pValue <<- V !|+ pAttribute <<- A)
>                     <<- \((attr,op),val) -> op attr val

> pCmpOp =
>       keyword "<=" <<- (\_-> Le)
>  !|+  keyword ">=" <<- (\_-> Ge)
>  !|+  keyword "=" <<- (\_-> Eq)
>  !|+  keyword "/=" <<- (\_-> Ne)
>  !|+  keyword "<"  <<- (\_-> Evarela.Lt)
>  !|+  keyword ">"  <<- (\_-> Evarela.Gt)

> pAttribute = pName





> tablesFromFile file = do
>  catch
>    (do
>      cs <- readFile file
>      return $ pRelDataDefs cs
>    )
>    (\e-> do
>      putStrLn ("cannot load file "++file++" "++show (e::SomeException))
>      return []
>    )

> printResult  (Left message) = putStrLn message
> printResult  (Right (typ,rows)) = do
>   putStrLn$show typ
>   sequence_$map  (putStrLn.show) rows


> moin file = do
>   putStrLn logo
>   ts <- tablesFromFile file
>   if (null ts) then (print "parse error")>>repl [][]
>   else do
>    parsedFile ts

> parsedFile ts = do
>    let rels  = fst$head ts
>    let rest  = snd$head ts
>    when (not$null rest) (putStrLn rest)
>    repl rels []





> repl rels exprs = do
>   maybeline <- readline "> "
>   case maybeline of
>     Nothing     -> repl rels exprs
>     Just ":q"   -> putStrLn "good bye!" >> return ()
>     Just "/exit"-> putStrLn "good bye!" >> return ()
>     Just "/help"-> putStr help >> repl rels exprs
>     Just ":?"   -> putStr help >> repl rels exprs
>     Just ":l"   -> putStrLn (show$map fst rels) >> repl rels exprs
>     Just "/list"-> putStrLn (show$map fst rels) >> repl rels exprs
>     Just ":qs"  -> putStrLn (show$map fst exprs) >> repl rels exprs
>     Just line   -> do
>      if isPrefixOf ":load" line
>      then do
>        ts <- tablesFromFile$ dropWhile (== ' ')$dropWhile (/= ' ') line
>        if (null ts) then (putStrLn "parse error") >> repl [][]
>        else parsedFile ts
>      else do
>       addHistory line
>       let query = pQueryDef line
>       if (not$null query)
>       then do
>         let ((q,_):_) = query
>         repl rels (q:exprs)
>       else do
>         let expr = pQuery line
>         let okay = not$null expr
>         if okay then do
>                   printParseRest (head expr)
>                   printResult (eval rels exprs (fst$head expr))
>                 else print ("cannot parse query: "++line)
>         repl rels exprs

> printParseRest (_,rs)
>  |not$null$filter (not.isSpace) rs = putStrLn ("unparsed string: "++rs)>>return ()
>  |otherwise = return ()





> logo =
>  "      ________ _    __  __\n\
>   \     /  _____/| |  / / /  \\\n\
>   \    /  /__    | | / / / /\\ \\\n\
>   \   /  ___/    | |/ / / /__\\ \\\n\
>   \  /  /____    | / / / _____  \\\n\
>   \ /_______/    |__/ /_/     \\__\\\n\
>   \  Evaluiere Relationale Algebra\n\
>   \  Programmende mit :q oder /exit \n\
>   \  Hilfe mit :? oder /help \n"





> help ="\n<expr>            Werte Ausdruck <expr> aus.\n\
>       \<name> := <expr>  Definiere gespeicherte Query <name> als <expr>.\n\
>       \:l                Liste die bekannten Relationen auf.\n\
>       \:load <filename>  Neue Datei mit definierten  Relationen laden.\n\
>       \:qs               Liste die gespeicherten Queries auf.\n\
>       \:?                Diese Hilfe anzeigen.\n\
>       \:q                Beende die Repl.\n"


