module Main where
import OurParserLib
import Evarela
import HaskellUnit(xmlEsc,replace)

import Control.Monad
import Control.Exception
import System.IO
import System.Environment
import Data.List(isPrefixOf,sort)
import Data.Char

main = do
  (solution:args) <- getArgs
  hSetEncoding stdout utf8
  Main.moin solution args

moin solution tests = sequence$map (processTestFile  solution) tests

processTestFile  solutionFile testFile = do
  putStrLn "<?xml version=\"1.0\" ?>"
  putStrLn "<testsuite>"
  catch
    (do
      cs1 <- readFile testFile
      cs2 <- readFile solutionFile
      let ts = uncomment cs1
      let os = uncomment cs2
      let sol = pSkript os
      let tests = pTests ts
      if null sol
        then (failTest "Kompilierung" "Lösung kann nicht geparst werden")
      else if null tests
        then (failTest "Kompilierung" "Tests können nicht geparst werden")
      else let
        testRest = parserRest$head tests
        solRest = parserRest$head sol
        in if not$null solRest
             then (failTest "Kompilierung" ("Lösung kann nicht komplett werden. Unparsed data: "++(snd$head sol)))
           else if not$null testRest
             then (failTest "Kompilierung" ("Tests können nicht komplett werden. Unparsed data: "++(snd$head tests)))
           else runTestSuite (fst$head sol) (fst$head tests)
    )
    (\e-> failTest "Kompilierung" (show (e::SomeException)))
  putStrLn "</testsuite>"



runTestSuite sol (defs,tests) = do
  xs <- sequence$map (runSingleTest sol defs) tests
  return ()

runSingleTest (_,sol) defs (T n rel) = do
  let result = eval defs sol (Relation n)
  res <- evalResult  result rel
  maybe
      (do
          putStrLn ("  <testcase name=\""++n++"\">")
          hFlush stdout
          putStrLn "  </testcase>"
          hFlush stdout
          return True)
      (\failMessage -> do
          putStrLn ("  <testcase name=\""++n++"\">")
          putStrLn ("    <failure message=\"siehe Details\"><![CDATA[")
          putStrLn (failMessage)
          putStrLn "    ]]></failure>"
          putStrLn "  </testcase>"
          return False)
      res


evalResult (Left msg)  _ = return (Just msg)
evalResult (Right expected) actual = do
  catch
    (if (relEq expected actual)
      then return Nothing
      else return (Just ( ("expected:"++show expected++" but was: "++show actual++""))))
    (\e->return (Just (replace "\n" " "("Exception in evaluation: "++ (show (e :: SomeException))))))


relEq (ts1,rss1) (ts2,rss2) = ts1==ts2 && sort rss1 == sort rss2

parserRest rs = filter (not.isSpace)$snd rs


failTest name fail = do
  putStrLn ("  <testcase name=\""++name++"\">")
  putStrLn ("    <failure message=\"siehe Details\">")
  putStrLn fail
  putStrLn "    </failure>"
  putStrLn "  </testcase>"
  return ()

comment ="--"

uncomment xs = unlines$map (skipComment comment)$lines (xs++"\n")

skipComment cmt [] = []
skipComment cmt ys@(x:xs)
  |isPrefixOf cmt ys = []
  |otherwise = x:skipComment cmt xs

data Test = T Ident Rel

pTestDataDef =
      (keyword "def"     -+- keyword "test" -+- pName  <<- \((_,_),n) -> n)
   -+-(keyword "expected"-+- pRelDef        -+- pTable <<- \((_,ts),rs)->(snd ts,rs))
             <<- \(n,rel) -> T n rel

pTestDataDefs = zeroToN(ignoreSpace pTestDataDef)

pTests :: Parser Char ([(Ident,Rel)],[Test])
pTests =pRelDataDefs +-  pTestDataDefs
