module Main where
import OurParserLib
import Turing
import HaskellUnit(xmlEsc,replace)

import Control.Monad
import Control.Exception
import System.IO
import System.Environment
import Data.List(isPrefixOf,sort)
import Data.Char
import Data.Maybe

main = do
  (solution:test:_) <- getArgs
  Main.moin solution test

moin solution test = processTestFile  solution test

processTestFile  solutionFile testFile = do
  putStrLn "<?xml version=\"1.0\" ?>"
  putStrLn "<testsuite>"
  catch
    (do
      cs1 <- readFile testFile
      cs2 <- readFile solutionFile
      let ts = uncomment cs1
      let os = uncomment cs2
      let sol = pProg os
      let tests = pTests ts
      if null sol
        then (failTest "Kompilierung" "Lösung kann nicht geparst werden")
      else if null tests
        then (failTest "Kompilierung" "Tests können nicht geparst werden")
      else let
        testRest = parserRest$head tests
        solRest = parserRest$head sol
        in if not$null solRest
             then (failTest "Kompilierung" ("Lösung kann nicht komplett werden. Unparsed data: "++(snd$head sol)))
           else if not$null testRest
             then (failTest "Kompilierung" ("Tests können nicht komplett werden. Unparsed data: "++(snd$head tests)))
           else runTestSuite (fst$head sol) (fst$head tests)
    )
    (\e-> failTest "Kompilierung" (show (e::SomeException)))
  putStrLn "</testsuite>"


runTestSuite solution tests = do
  xs <- sequence$map (runSingleTest solution) tests
  return ()


runSingleTest solution (name,input,output) = do
  let (TM _ _ result) = fromJust$last$run $TM solution "s0" input
  res <- evalResult output result
  maybe
      (do
          putStrLn ("  <testcase name=\""++name++"\">")
          hFlush stdout
          putStrLn "  </testcase>"
          hFlush stdout
          return True
          )
      (\failMessage -> do
          putStrLn ("  <testcase name=\""++name++"\">")
          putStrLn ("    <failure message=\""++failMessage++"\"><![CDATA[")
          putStrLn (failMessage)
          putStrLn "    ]]></failure>"
          putStrLn "  </testcase>"
          return False
          )
      res



evalResult:: Band Char -> Band Char  -> IO (Maybe String)
evalResult expected solution = do
  let e = getBand expected
  let s = getBand solution
  catch
    (if (e  == s)
      then return Nothing
      else return (Just ("expected: "++show e++ " actual: "++show s))
    )
    (\e->return (Just (replace "\n" " "("Exception in evaluation: "++ (show (e :: SomeException))))))




parserRest rs = filter (not.isSpace)$snd rs


failTest name fail = do
  putStrLn ("  <testcase name=\""++name++"\">")
  putStrLn ("    <failure message=\"(Fehler in Testausfuehrung "++fail++"\">")
  putStrLn fail
  putStrLn "    </failure>"
  putStrLn "  </testcase>"


comment ="--"

uncomment xs = unlines$map (skipComment comment)$lines (xs++"\n")

skipComment cmt [] = []
skipComment cmt ys@(x:xs)
  |isPrefixOf cmt ys = []
  |otherwise = x:skipComment cmt xs

pTests = oneToN (ignoreSpace pTest)

pTest = pTestName -+- pEingabe -+- pAusgabe <<-( \((a,b),c)->(a,b,c))

pTestName    = keyword "test"   -+- terminal ':' -+- pName <<- snd
pEingabe = keyword "input"  -+- terminal ':' -+- pBand <<- snd
pAusgabe = keyword "output" -+- terminal ':' -+- pBand <<- snd

pBand = pBand2 +- terminal ' ' -+- pVal +- terminal ' ' -+- pBand2
  <<-  (\((((ls,_),c),_),rs)->Band (reverse ls, c, rs))

pBand2 = zeroToN pVal -- <<- (\xs -> Band ([],Blank,xs))

getBand (Band (ls,c,rs)) = rmblank (reverse ls++c:rs)
  where
    rmblank  = reverse.dropWhile (== Blank).reverse.dropWhile (== Blank)
