



> module Turing where
> import Data.Maybe (isJust)
> import Data.Char

> import OurParserLib





> data TuringMaschine symbol zustand
>   = TM (Program symbol zustand) zustand (Band symbol)
>     deriving (Show,Eq)





> data Movement = L| R| N deriving (Eq, Show, Read)





> type Program symbol zustand = [Transition symbol zustand]





> type Transition symbol zustand
>   = ((zustand,Entry symbol),(Entry symbol,zustand,Movement))





> newtype Band symbol = Band ([Entry symbol],Entry symbol,[Entry symbol]) deriving Eq





> instance Show a => Show (Band a) where
>  show (Band (ls,c,rs))
>    = concat (map show $reverse ls)++"|"++show c++"|"++concat (map show rs)





> data Entry symbol = Blank| Symbol symbol deriving (Eq)





> instance Show a => Show (Entry a) where
>  show Blank = "_"
>  show (Symbol s) = show s





> transitionFromString :: String -> Transition Int String
> transitionFromString cs =  ((s1,readSymbolAsInt c1),(readSymbolAsInt c2,s2,read m))
>   where
>    [s1,c1,c2,s2,m] = words cs
>    readSymbolAsInt "_" = Blank
>    readSymbolAsInt x = Symbol ((read x)::Int)





> tmDoppel
>  = TM
>      (map transitionFromString
>        [ "1 1 _ 2 R", "1 _ _ 6 N", "2 1 1 2 R", "2 _ _ 3 R", "3 1 1 3 R"
>        , "3 _ 1 4 L", "4 1 1 4 L", "4 _ _ 5 L", "5 1 1 5 L", "5 _ 1 1 R" ]
>      )
>      "1"





> class ToLaTeX a where
>   toLaTeX :: a -> String





> instance (Show symbol,Show zustand,Eq zustand,Eq symbol)
>        => ToLaTeX (TuringMaschine symbol zustand) where





>   toLaTeX (TM prog zustand band@(Band (_,c,_)) ) =
>       "\n\\begin{tikzpicture}"
>     ++"\n\\tikzstyle{every path}=[very thick]"
>     ++"\n\\edef\\sizetape{0.5cm}"
>     ++"\n\\tikzstyle{tmtape}=[draw,minimum size=\\sizetape]"
>     ++ arrows
>     ++ toLaTeX band
>     ++ lesekopf
>     ++ next
>     ++"\n\\end{tikzpicture}\n\n\n"
>     where
>       trans = lookup (zustand,c) prog
>       lesekopf =  ("\n\\node [tmhead,yshift=-.3cm] at (input.south) (head) "
>                    ++ (" {$q_{"++(filter (/= '\"')$show zustand)++"}$};"))
>       arrows =
>         maybe
>            "\n\\tikzstyle{tmhead}=[draw,minimum size=.3cm]"
>            (\(_,_,m) -> arrow m)
>            trans
>       arrow L = tmhead++"arrows={west:0.25cm}]"
>       arrow R = tmhead++"arrows={east:0.25cm}]"
>       arrow N = "\n\\tikzstyle{tmhead}=[draw,minimum size=.3cm]"
>
>       tmhead = "\n\\tikzstyle{tmhead}=[arrow box,draw,minimum size=.3cm,arrow box "
>
>       next =
>         maybe
>           ""
>           (\(s,q,m) ->
>              "\n\\node [minimum size=.1cm,xshift=.5cm] at (head.east) (nextstate)"
>             ++"\n {$(q_{"++(filter (/= '\"')$show zustand)++"},$"++sym s++"$)$};")
>           trans
>       sym Blank = "_"
>       sym s = toLaTeX s





> instance (Show symbol) => ToLaTeX (Band symbol) where
>   toLaTeX (Band (ls,c,rs)) =
>    "\n\\begin{scope}[start chain=1 going right,node distance=-0.15mm]"
>    ++"\n\\node [on chain=1,tmtape,draw=none] {$\\ldots$};"
>    ++(concat$map showInBand$reverse ls)
>    ++"\n\\node [on chain=1,tmtape] (input) {"++toLaTeX c++"};"
>    ++(concat$map showInBand rs)
>    ++"\n\\node [on chain=1,tmtape,draw=none] {$\\ldots$};"
>    ++"\n\\end{scope}"
>     where
>       showInBand symbol = "\n\\node [on chain=1,tmtape] {"++(toLaTeX symbol )++"};"





> instance (Show symbol) => ToLaTeX (Entry symbol) where
>   toLaTeX Blank      = ""
>   toLaTeX (Symbol s) = show s





> move :: Movement -> Band symbol -> Band symbol
> move N band = band
> move L (Band (l:ls,c,rs)) = Band (ls,l,c:rs)
> move R (Band (ls,c,r:rs)) = Band (c:ls,r,rs)
> move L (Band ([],c,rs)) = Band ([],Blank,c:rs)
> move R (Band (ls,c,[])) = Band (c:ls,Blank,[])





> step :: (Eq zustand, Eq symbol)
>   => TuringMaschine symbol zustand -> Maybe (TuringMaschine symbol zustand)
> step (TM prog z (Band (ls,c,rs))) =  fmap t trans
>  where
>   trans = lookup (z,c) prog
>   t (s,z',m) = TM prog z' (move m (Band (ls,s,rs)))





> stepmaybe :: (Eq zustand, Eq symbol)
>   => Maybe (TuringMaschine symbol zustand) -> Maybe (TuringMaschine symbol zustand)
> stepmaybe = flatten.fmap step
>  where
>   flatten (Just Nothing) = Nothing
>   flatten (Just j) = j
>   flatten Nothing = Nothing





> run :: (Eq zustand, Eq symbol)
>   => TuringMaschine symbol zustand -> [Maybe (TuringMaschine symbol zustand)]
> run = takeWhile isJust.iterate stepmaybe.Just





> doppel =
>     map show$map (\(Just(TM _ _ b))->b)
>    $run (tmDoppel (Band ([],Symbol 1,map Symbol[1, 1, 1, 1])))





> tmAdd
>  = TM
>    (map transitionFromString
>     ["0 _ _ 0 R","0 1 1 1 R", "1 _ 1 2 R"   ,"1 1 1 1 R" ,"2 _ _ 3 R" ,"2 1 1 2 R"
>     ,"3 _ _ 3 L","3 1 _ 4 L", "4 _ _ halt N" ,"4 1 1 4 L"])
>     "0"





> add2x3 = run (tmAdd (Band ([],Blank,map Symbol [1, 1]++Blank:map Symbol [1, 1, 1])))





> showAdd2x3 = map show$map (\(Just(TM _ _ b))->b) add2x3





> latexAdd2x3 = concat$map (\(Just tm)-> toLaTeX tm) add2x3





> tmMult
>  = TM
>    (map transitionFromString
>     ["s0 _ _ s1 R"    ,"s0 1 _ s2 R"   ,"s1 _ _ s14 R"   ,"s1 1 _ s2 R"
>     ,"s2 _ _ s3 R"    ,"s2 1 1 s2 R"   ,"s3 _ _ s15 L"   ,"s3 1 _ s4 R"
>     ,"s4 _ _ s5 R"    ,"s4 1 1 s4 R"   ,"s5 _ 1 s6 L"    ,"s5 1 1 s5 R"
>     ,"s6 _ _ s7 L"    ,"s6 1 1 s6 L"   ,"s7 _ 1 s9 L"    ,"s7 1 1 s8 L"
>     ,"s8 _ 1 s3 R"    ,"s8 1 1 s8 L"   ,"s9 _ _ s10 L"   ,"s9 1 1 s9 L"
>     ,"s10 _ _ s12 R"  ,"s10 1 1 s11 L" ,"s11 _ _ s0 R"   ,"s11 1 1 s11 L"
>     ,"s12 _ _ s12 R"  ,"s12 1 _ s13 R" ,"s13 _ _ halt N" ,"s13 1 _ s13 R"
>     ,"s14 _ _ halt N" ,"s14 1 _ s14 R" ,"s15 _ _ s16 L"  ,"s15 1 _ s15 L"
>     ,"s16 _ _ halt N" ,"s16 1 _ s16 L"])
>     "s0"





> mult2x3 = run (tmMult (Band ([],Blank,map Symbol [1,1]++Blank:map Symbol [1,1,1])))





> tmFac
>  = TM
>    (map transitionFromString
>     ["s0 _ _ s1 R"
>     ,"s1 1 1 s2 R","s1 _ _ s56 N","s2 _ _ s12 L","s2 1 _ s3 R"
>     ,"s3 1 1 s3 R","s3 _ _ s4 R"
>     ,"s4 1 1 s3 R","s4 _ 1 s5 L"
>     ,"s5 _ _ s6 L","s5 1 1 s5 L"
>     ,"s6 _ 1 s7 R","s6 1 1 s6 L"
>     ,"s7 _ _ s1 R","s7 1 _ s8 R"
>     ,"s8 _ _ s9 R","s8 1 1 s8 R"
>     ,"s9 _ 1 s5 L","s9 1 1 s9 R"
>     ,"s10 1 1 s10 R","s10 _ _ s11 R","s11 1 1 s11 R","s11 _ _ s6 L"
>     ,"s12 1 _ s13 L","s12 _ _ s14 L"
>     ,"s13 1 1 s13 L","s13 _ _ s14 L"
>     ,"s14 _ _ s15 R","s14 1 1 s13 L"
>     ,"s15 _ _ s31 R","s15 1 1 s31 N"
>     ,"s31 _ _ s98 R","s31 1 _ s16 R"
>     ,"s16 _ _ s17 R","s16 1 1 s16 R"
>     ,"s17 _ _ s98 L","s17 1 _ s18 R"
>     ,"s18 _ _ s19 R","s18 1 1 s18 R"
>     ,"s19 _ _ s20 N","s19 1 1 s18 R"
>     ,"s20 _ 1 s21 L","s20 1 1 s20 R"
>     ,"s21 _ _ s44 L","s21 1 1 s21 L"
>     ,"s44 _ _ s22 L","s44 1 1 s21 L"
>     ,"s22 _ _ s23 R","s22 1 1 s21 L"
>     ,"s23 _ _ s45 R","s45 _ _ s24 R"
>     ,"s24 _ _ s25 R","s24 1 1 s24 R"
>     ,"s25 _ 1 s40 R","s25 1 1 s25 R"
>     ,"s26 _ _ s27 L","s26 1 1 s26 L"
>     ,"s27 _ _ s29 R","s27 1 1 s28 L"
>     ,"s28 _ _ s15 R","s28 1 1 s28 L"
>     ,"s29 _ _ s29 R","s29 1 _ s30 R"
>     ,"s30 _ _ halt N","s30 1 _ s30 R"
>     ,"s40 _ _ s50 N","s40 1 _ s41 R"
>     ,"s41 _ _ s42 R","s41 1 1 s41 R"
>     ,"s42 _ _ s43 L","s42 1 1 s41 R"
>     ,"s43 _ 1 s21 L","s50 1 _ s51 L"
>     ,"s50 _ _ s52 L","s51 1 1 s51 L"
>     ,"s51 _ _ s52 L","s52 _ _ s53 R"
>     ,"s52 1 1 s51 L","s53 _ _ s62 R"
>     ,"s62 1 1 s63 R","s63 _ _ s64 L"
>     ,"s63 1 1 s54 L","s64 _ _ s64 R"
>     ,"s64 1 _ s65 R","s65 _ _ s66 R"
>     ,"s66 _ _ s90 R","s66 1 1 s90 R"
>     ,"s90 _ _ s91 L","s90 1 1 s67 L"
>     ,"s67 1 _ s68 R","s68 _ _ s69 R"
>     ,"s68 1 1 s68 R","s69 _ _ s70 L"
>     ,"s69 1 1 s68 R","s70 _ 1 s71 L"
>     ,"s71 _ _ s72 L","s71 1 1 s71 L"
>     ,"s72 _ _ s73 R","s72 1 1 s71 L"
>     ,"s73 _ _ s66 R","s91 1 _ s92 R"
>     ,"s92 _ _ s93 R","s92 1 1 s92 R"
>     ,"s93 _ _ s94 L","s93 1 1 s92 R"
>     ,"s94 _ 1 s95 L","s95 _ _ s96 L"
>     ,"s95 1 1 s95 L","s96 _ _ s97 R"
>     ,"s96 1 1 s95 L","s97 _ _ s15 R"
>     ,"s98 _ 1 s99 N"
>     ,"s99 1 1 s99 L","s99 _ _ halt N"
>     ,"s54 _ _ s15 R","s54 1 _ s55 R"
>     ,"s55 _ _ s56 R","s55 1 1 s55 R"
>     ,"s56 _ _ halt N","s56 1 _ s57 R"
>     ,"s57 _ _ s58 R","s57 1 1 s57 R"
>     ,"s58 _ _ s59 L","s58 1 1 s57 R"
>     ,"s59 _ 1 s21 L"
>     ,"s59 1 1 s60 R"])
>     "s0"





> fac4 = run (tmFac (Band ([],Blank,map Symbol [1, 1, 1, 1])))





> pName =    (satisfy (\x-> isAlpha x || elem x "_"))
>         +- (zeroToN
>             $satisfy (\x ->(isAlpha x||isDigit x||elem x "_") && (not$isSpace x)))
>          <<- (\(x,xs)->x:xs)


> pState = pName

> pDirection =
>       (terminal 'R' <<- (\_ -> R))
>   !|+ (terminal 'L' <<- (\_ -> L))
>   !|+ (terminal 'N' <<- (\_ -> N))

> pVal =
>          (terminal '_' <<- \_ -> Blank)
>     !|+  satisfy (not.isSpace) <<- \n -> Symbol n

> pTransition :: Parser Char (Transition Char String)
> pTransition = (pState +- (ignoreInlineSpace pVal)) +-
>               ((ignoreInlineSpace pVal)+-(ignoreInlineSpace pState)+-(ignoreInlineSpace pDirection)
>                    <<- (\((a,b),c)->(a,b,c)))

> ignoreInlineSpace p = \xs -> p (dropWhile (\x->isSpace x && not (x=='\n')) xs)

> pProg :: Parser Char (Program Char String)
> pProg = sepByOne (ignoreSpace pTransition) (ignoreInlineSpace$terminal '\n')


