package de.hsrm.sls.eva.request;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@JsonDeserialize(builder = EvalRequest.Builder.class)
public class EvalRequest {
    private final String stackRef;
    private final List<EvaFile> solutionFiles;
    private final List<EvaFile> includeFiles;
    private final List<EvaFile> testFiles;
    private final RequestedCapability capability;
    private final Map<String, String> params;

    private EvalRequest(String stackRef, List<EvaFile> solutionFiles, List<EvaFile> includeFiles,
        List<EvaFile> testFiles, RequestedCapability capability, Map<String, String> params) {
        this.stackRef = stackRef;
        this.solutionFiles = solutionFiles;
        this.includeFiles = includeFiles;
        this.testFiles = testFiles;
        this.capability = capability;
        this.params = params;
    }

    public String getStackRef() {
        return stackRef;
    }

    public List<EvaFile> getSolutionFiles() {
        return solutionFiles;
    }

    public List<EvaFile> getIncludeFiles() {
        return includeFiles;
    }

    public List<EvaFile> getTestFiles() {
        return testFiles;
    }

    public RequestedCapability getCapability() {
        return capability;
    }

    public Map<String, String> getParams() {
        return params;
    }

    @JsonPOJOBuilder(withPrefix = "set")
    public static class Builder {
        private String stackRef;
        private List<EvaFile> solutionFiles = new ArrayList<>();
        private List<EvaFile> includeFiles = new ArrayList<>();
        private List<EvaFile> testFiles = new ArrayList<>();
        private RequestedCapability capability;
        private Map<String, String> params = new HashMap<>();

        public Builder setStackRef(String stackRef) {
            this.stackRef = stackRef;
            return this;
        }

        public Builder setSolutionFiles(List<EvaFile> solutionFiles) {
            this.solutionFiles = solutionFiles;
            return this;
        }

        public Builder setIncludeFiles(List<EvaFile> includeFiles) {
            this.includeFiles = includeFiles;
            return this;
        }

        public Builder setTestFiles(List<EvaFile> testFiles) {
            this.testFiles = testFiles;
            return this;
        }

        public Builder setCapability(RequestedCapability capability) {
            this.capability = capability;
            return this;
        }

        public Builder setParams(Map<String, String> params) {
            this.params = params;
            return this;
        }

        public EvalRequest build() {
            return new EvalRequest(stackRef, solutionFiles, includeFiles, testFiles, capability, params);
        }
    }
}
