import typing
from typing import List

from eva.pltform.evaluation.core import EvaFile, EvalResult, EvaluatorOptions
from pydantic import BaseModel


class RequestedCapability(BaseModel):
    compile: bool = False
    test: bool = False
    style: bool = False


class EvalRequest(BaseModel):
    stack_ref: str
    solution_files: List[EvaFile] = []
    include_files: List[EvaFile] = []
    test_files: List[EvaFile] = []

    # if not specified by clients, all supported capabilities (except coverage) will be used
    capability: typing.Optional[RequestedCapability] = None
    params: dict[str, str] = {}


class SysConfig(BaseModel):
    rt: dict = {}
    evaluator_opts: EvaluatorOptions


class EvalResponse(BaseModel):
    success: bool = False
    message: str = ''
    result: typing.Optional[EvalResult] = None
    version: str
    sysconf: SysConfig
