import datetime
from typing import Optional

import requests
from pydantic import BaseModel
from requests import HTTPError


class Environment(BaseModel):
    id: str
    description: str
    maintainer: str


class EvaluatorCapability(BaseModel):
    compile: bool
    style: bool
    test: bool


class EvaluatorDeclaration(BaseModel):
    id: str
    capability: EvaluatorCapability
    supported_envs: Optional[str]
    excluded_envs: Optional[str]
    default_env: Optional[str]


class EvaluationStack(BaseModel):
    environment: Environment
    evaluator: EvaluatorDeclaration
    ref: str


class GetStackResponse(BaseModel):
    stack: EvaluationStack
    last_update: datetime.datetime


class RegistryClient:
    TIMEOUT = 20

    def __init__(self, registry_url):
        self.registry_url = registry_url
        self.evaluators = None

    def get_stack(self, ref) -> GetStackResponse:
        try:
            url = f'{self.registry_url}/stacks/{ref}'
            response = requests.get(url, timeout=self.TIMEOUT)
            response.raise_for_status()
            return GetStackResponse(**response.json())
        except HTTPError as e:
            if e.response.status_code == 404:
                raise RuntimeError(f'no stack found for ref {ref}') from e

            raise RuntimeError('Querying stacks failed') from e
