from typing import List

import streamlit as st
from eva.playground.dispatcher.dto import EvaFile
from eva.playground.state import FilesState
from streamlit_ace import st_ace

editor_conf = {
    'auto_update': False,
    'font_size': 16,
    'theme': 'vibrant_ink',
    'language': 'java',
    'show_gutter': False,
}


def file_list(files: List[EvaFile], id_prefix, on_click):
    for idx, include_file in enumerate(files):
        col1, col2 = st.columns([1, 4])
        with col1:
            include_name = st.text_input(
                'File name',
                include_file.path,
                key=f'{id_prefix}{idx}n',
                label_visibility='collapsed',
            )
            include_file.path = include_name
        with col2:
            st.button(':x:', key=f'{id_prefix}{idx}d', on_click=files.remove, args=(include_file,))

        # key option in st_ace is bugged (editor does not update on session change)
        # so generate a new key every time the content changes
        content = st_ace(
            include_file.content,
            key=f'{id_prefix}{idx}{hash(include_file.content)}a',
            **editor_conf,
        )
        include_file.content = content

    st.button('Add', key=f'{id_prefix}b', on_click=on_click)


def files_tabset(files: FilesState):
    st.warning(
        'Please press CTRL + Enter if you change the file contents. '
        'Changes are not picked up automatically.'
    )

    tab1, tab2, tab3 = st.tabs(['Solution', 'Includes', 'Tests'])
    with tab1:
        content = st_ace(files.solution_file.content, **editor_conf)
        files.solution_file.content = content

    with tab2:
        file_list(files.include_files, 'i', files.add_include)

    with tab3:
        file_list(files.test_files, 't', files.add_test)
