from dataclasses import dataclass, field

import pandas as pd
import streamlit as st
from eva.playground.registry import EvaluationStack
from eva.playground.stef import StefTask


@dataclass
class Form:
    compile: bool = False
    test: bool = False
    coverage: bool = False
    coverage_tool: str = None
    style: bool = False
    log_level: str = 'DEBUG'
    params: dict = field(default_factory=lambda: {})


def param_editor(params: dict):
    data = [{'key': k, 'value': v} for k, v in params.items()]
    df = pd.DataFrame(data, columns=['key', 'value'])

    st.write('Params')
    edited_df = st.data_editor(df, num_rows='dynamic', hide_index=True)
    return {r['key']: r['value'] for idx, r in edited_df.iterrows()}


def request_form(task: StefTask, stack: EvaluationStack):
    form = Form()

    st.write('Select steps to execute')

    if stack.evaluator.capability.compile:
        form.compile = st.checkbox('Compile', value=True)

    if stack.evaluator.capability.style:
        form.style = st.checkbox('Style', value=True)

    if stack.evaluator.capability.test:
        form.test = st.checkbox('Test', value=True)

    if stack.evaluator.id in ['java', 'javaie']:
        form.coverage = st.checkbox('Coverage')

        if form.coverage:
            coverage_opts = [('jacoco', 'JaCoCo'), ('clover', 'OpenClover')]
            tool = st.selectbox('Coverage Tool', coverage_opts, format_func=lambda x: x[1])
            form.coverage_tool = tool[0]

    params = param_editor(task.meta.evaluator_params)
    form.params = params

    log_levels = ['DEBUG', 'INFO']
    log_level = st.selectbox('Log Level', log_levels)
    form.log_level = log_level

    return st.button('Run', type='primary'), form
