import streamlit as st
from eva.playground.components.result.coverage import coverage_list
from eva.playground.components.result.style import style_table
from eva.playground.components.result.test import test_table
from eva.playground.state import EvalState


def render_result(evaluation: EvalState):
    if evaluation.has_result():
        if evaluation.result.compilation:
            st.subheader(f'Compiler: {"✅" if evaluation.result.compilation.compiling else "❌"}')

            st.write('Output:')
            st.code(
                evaluation.result.compilation.output
                if evaluation.result.compilation.output
                else 'Keine Ausgabe vorhanden'
            )

        if evaluation.result.test:
            st.subheader(
                f'Tests: {evaluation.result.test.passed}/{evaluation.result.test.testcases}'
            )
            test_table(evaluation.result)

            st.write('Output:')
            st.code(
                evaluation.result.test.output
                if evaluation.result.test.output
                else 'Keine Ausgabe vorhanden'
            )

        if evaluation.result.style:
            st.subheader(f'Style: {len(evaluation.result.style.report.errors)} errors')
            style_table(evaluation.result)

        if evaluation.result.test and evaluation.result.test.coverage:
            coverage_list(evaluation.req, evaluation.result)

    if evaluation.has_error():
        st.error(evaluation.error)

    if evaluation.has_logs():
        with st.expander('Logs'):
            st.code(evaluation.logs)
