import html

import streamlit as st
import streamlit.components.v1 as components
from eva.playground.dispatcher.cexf import Coverage, LineDecision, LineType
from eva.playground.dispatcher.dto import EvaFile, EvalRequest
from eva.playground.dispatcher.result import EvalResult


def highlight_coverage(file: EvaFile, coverage: Coverage):
    highlights = {}
    for testcase in coverage.testsuite.testcases:
        file_coverage = next(iter(f for f in testcase.files if f.name == file.path), None)
        if file_coverage is None:
            continue

        file_lines = file.content.split('\n')
        for i, line in enumerate(file_lines):
            l = next(
                iter([l for l in file_coverage.lines if l.type == LineType.LINE and l.nr - 1 == i]),
                None,
            )

            color = 'transparent'
            if l is not None:
                if l.decision == LineDecision.FULL:
                    color = '#b8ffb8'
                elif l.decision == LineDecision.PARTIAL:
                    color = '#ffff9a'
                elif l.decision == LineDecision.MISSED:
                    color = '#ffb3b3'

            new_line = (
                f'<div style="background-color: {color}; ">{str(i+1)} {html.escape(line)}</div>'
            )
            file_lines[i] = new_line

        html_code = (
            '<pre style="font-size: 12px; background-color: white;padding: 10px;color: black">'
            + ''.join(file_lines)
            + '</pre>'
        )
        highlights[testcase.id] = html_code

    return highlights


def coverage_list(req: EvalRequest, result: EvalResult):
    st.subheader(f'Coverage: {req.solution_files[0].path} ({result.test.coverage.tool})')
    highlights = highlight_coverage(req.solution_files[0], result.test.coverage)
    for testcase, content in highlights.items():
        with st.expander(testcase):
            components.html(content, height=400, scrolling=True)
