import pandas as pd
import streamlit as st
from eva.playground.dispatcher.result import EvalResult
from eva.playground.dispatcher.trex import (
    ErrorResult,
    FailureResult,
    PendingResult,
    StartedResult,
    SuccessResult,
    Testcase,
    TimeoutResult,
)


def get_status(testcase: Testcase):
    if isinstance(testcase.result, SuccessResult):
        return 'success'
    if isinstance(testcase.result, FailureResult):
        return 'failure'
    if isinstance(testcase.result, ErrorResult):
        return 'error'
    if isinstance(testcase.result, (StartedResult, TimeoutResult)):
        return 'timeout'
    if isinstance(testcase.result, PendingResult):
        return 'pending'
    return ''


def get_status_icon(testcase: Testcase):
    if isinstance(testcase.result, SuccessResult):
        return '✅'
    if isinstance(testcase.result, PendingResult):
        return '⋯'
    return '❌'


def test_table(result: EvalResult):
    df = pd.DataFrame(
        {
            'id': [t.id for t in result.test.testsuite.testcases],
            'status': [get_status(t) for t in result.test.testsuite.testcases],
            'status_icon': [get_status_icon(t) for t in result.test.testsuite.testcases],
            'json': [t.result.model_dump_json(indent=4) for t in result.test.testsuite.testcases],
        }
    )
    st.dataframe(
        df,
        column_config={'id': 'Testcase ID', 'status_icon': '', 'status': 'Status', 'json': ''},
        hide_index=True,
        use_container_width=True,
    )
