from enum import IntEnum
from typing import List

from pydantic import BaseModel


class LineDecision(IntEnum):
    NONE = 0
    MISSED = 1
    PARTIAL = 2
    FULL = 3


class LineType(IntEnum):
    LINE = 0
    INSTR = 1
    STMT = 2
    METHOD = 3
    COND = 4
    CLAZZ = 5


class Line(BaseModel):
    nr: int
    missed: int
    covered: int
    type: LineType
    decision: LineDecision


class File(BaseModel):
    name: str
    fqcn: str
    lines: List[Line]


class Testcase(BaseModel):
    id: str
    files: List[File]


class Testsuite(BaseModel):
    testcases: List[Testcase]


class Coverage(BaseModel):
    tool: str
    version: str
    testsuite: Testsuite
