from typing import Annotated, List, Literal, Optional, Union

from pydantic import BaseModel, Field


class Result(BaseModel):
    type: str


class SuccessResult(Result):
    type: Literal['success'] = 'success'


class ErrorResult(Result):
    type: Literal['error'] = 'error'
    details: str
    message: str


class FailureResult(Result):
    type: Literal['failure'] = 'failure'
    details: str
    message: str


class PendingResult(Result):
    type: Literal['pending'] = 'pending'


class StartedResult(Result):
    type: Literal['started'] = 'started'


class TimeoutResult(Result):
    type: Literal['timeout'] = 'timeout'
    timeout: int


ResultUnion = Annotated[
    Union[SuccessResult, ErrorResult, FailureResult, PendingResult, StartedResult, TimeoutResult],
    Field(discriminator='type'),
]


class Testcase(BaseModel):
    id: str
    name: str
    class_name: Optional[str]
    result: ResultUnion
    stdout: Optional[str]
    stderr: Optional[str]


class Testsuite(BaseModel):
    skipped: int = 0
    timestamp: Optional[str]
    time: Optional[str]

    allocated: Optional[int]
    free: Optional[int]

    stdout: Optional[str]
    stderr: Optional[str]
    testcases: List[Testcase]
