import requests
from eva.playground.registry.dto import GetStackResponse, StacksResponse
from requests import HTTPError


class RegistryClient:
    TIMEOUT = 20

    def __init__(self, registry_url):
        self.registry_url = registry_url

    def get_stacks(self) -> StacksResponse:
        url = f'{self.registry_url}/stacks'
        response = requests.get(url, timeout=self.TIMEOUT)
        response.raise_for_status()
        stacks_response = response.json()
        return StacksResponse(**stacks_response)

    def get_stack(self, ref) -> GetStackResponse:
        try:
            url = f'{self.registry_url}/stacks/{ref}'
            response = requests.get(url, timeout=self.TIMEOUT)
            response.raise_for_status()
            return GetStackResponse(**response.json())
        except HTTPError as e:
            if e.response.status_code == 404:
                raise RuntimeError(f'no stack found for ref {ref}') from e

            raise RuntimeError('Querying stacks failed') from e
