import datetime
from typing import List, Optional

from pydantic import BaseModel


class Environment(BaseModel):
    id: str
    description: str
    maintainer: str


class EvaluatorCapability(BaseModel):
    compile: bool
    style: bool
    test: bool


class EvaluatorDeclaration(BaseModel):
    id: str
    capability: EvaluatorCapability
    supported_envs: Optional[str]
    excluded_envs: Optional[str]
    default_env: Optional[str]


class EvaluationStack(BaseModel):
    environment: Environment
    evaluator: EvaluatorDeclaration
    ref: str


class StacksResponse(BaseModel):
    stacks: List[EvaluationStack]
    last_update: datetime.datetime


class GetStackResponse(BaseModel):
    stack: EvaluationStack
    last_update: datetime.datetime
