import os

import streamlit as st
from eva.playground.dispatcher import DispatcherClient
from eva.playground.registry import EvaluationStack, RegistryClient
from eva.playground.task import TaskManager

TTL = 300


@st.cache_resource(ttl=TTL)
def get_tasks(_dir_path):
    task_manager = TaskManager(_dir_path)
    return task_manager.load_tasks()


def get_dispatcher():
    rabbitmq_host = os.getenv('EVA_PG_RABBITMQ_HOST', 'localhost')
    c = DispatcherClient(rabbitmq_host)
    c.open()
    return c


def get_registry():
    registry_url = os.getenv('EVA_PG_REGISTRY_URL', 'http://localhost:7777')
    return RegistryClient(registry_url)


def has_capability(stack: EvaluationStack):
    return (
        stack.evaluator.capability.compile
        or stack.evaluator.capability.style
        or stack.evaluator.capability.test
    )


@st.cache_data(ttl=TTL)
def get_stacks(_registry: RegistryClient):
    response = _registry.get_stacks()
    return list(filter(has_capability, response.stacks))


@st.cache_data(ttl=TTL)
def get_stack(_registry: RegistryClient, ref: str):
    response = _registry.get_stack(ref)
    return response.stack
