import typing

import streamlit as st
from eva.playground.dispatcher.dto import EvaFile, EvalRequest
from eva.playground.dispatcher.result import EvalResult
from pydantic import BaseModel


class EvalState(BaseModel):
    req: EvalRequest = None
    result: EvalResult = None
    error: str = None
    logs: str = None

    def has_result(self):
        return self.result is not None and self.req is not None

    def has_error(self):
        return self.error is not None

    def has_logs(self):
        return self.logs is not None


class FilesState(BaseModel):
    solution_file: EvaFile = None
    include_files: typing.List[EvaFile] = []
    test_files: typing.List[EvaFile] = []

    def add_include(self):
        f = EvaFile(path='Unnamed.java', content='')
        self.include_files.append(f)

    def add_test(self):
        f = EvaFile(path='UnnamedTest.java', content='')
        self.test_files.append(f)


class State:
    KEY_FILES = 'files'
    KEY_EVAL = 'eval'
    KEY_REQUEST = 'request'

    def evaluation(self) -> EvalState:
        if self.KEY_EVAL not in st.session_state:
            st.session_state[self.KEY_EVAL] = EvalState()
        return st.session_state[self.KEY_EVAL]

    def files(self, task) -> FilesState:
        if self.KEY_FILES not in st.session_state:
            solution_file = [
                EvaFile(path=f.path, content=task.read_file(f))
                for f in task.meta.files
                if f.type == 'solution'
            ][0]
            include_files = [
                EvaFile(path=f.path, content=task.read_file(f))
                for f in task.meta.files
                if f.type == 'include'
            ]
            test_files = [
                EvaFile(path=f.path, content=task.read_file(f))
                for f in task.meta.files
                if f.type == 'test'
            ]
            st.session_state[self.KEY_FILES] = FilesState(
                solution_file=solution_file, include_files=include_files, test_files=test_files
            )

        return st.session_state.get(self.KEY_FILES)

    def clear_files(self):
        st.session_state.pop(self.KEY_FILES)


state = State()
