import os
import xml.etree.ElementTree as ET


class File:
    def __init__(self, **kwargs):
        self.type = kwargs.get('type')
        self.path = kwargs.get('path')
        self.public = kwargs.get('public')


class Meta:
    def __init__(self, **kwargs):
        self.name = kwargs.get('name')
        self.subato_id = kwargs.get('subato_id')
        self.evaluator = kwargs.get('evaluator')
        self.evaluator_params = kwargs.get('evaluator_params')
        self.description = kwargs.get('description')
        self.lang = kwargs.get('lang')
        self.attempts = kwargs.get('attempts')
        self.files = kwargs.get('files')


class TextFile:
    def __init__(self, **kwargs):
        self.ext = kwargs.get('ext')


class StefTask:
    def __init__(self, root, meta, path, text_files):
        self.root = root
        self.meta = meta
        self.path = path
        self.text_files = text_files

    def read_file(self, file: File):
        with open(os.path.join(self.root, 'files', file.type, file.path), encoding='utf-8') as f:
            return f.read()

    def __eq__(self, other):
        return self.path == other.path


class StefParser:
    def parse(self, path):
        meta_parser = MetaParser()
        meta_file = os.path.join(path, 'meta.xml')
        if not os.path.exists(meta_file):
            return None

        meta = meta_parser.parse(meta_file)

        text_files = []

        text_html = os.path.join(path, 'text/text.html')
        if os.path.exists(text_html):
            text_files.append(TextFile(ext='html'))

        text_md = os.path.join(path, 'text/text.md')
        if os.path.exists(text_md):
            text_files.append(TextFile(ext='md'))

        return StefTask(root=path, meta=meta, path=path, text_files=text_files)


class MetaParser:
    def parse(self, path):
        tree = ET.parse(path)
        root_element = tree.getroot()

        name = self.parse_element_text(root_element, 'name')
        description = self.parse_element_text(root_element, 'description')
        evaluator = self.parse_element_text(root_element, 'evaluator')
        params = self.parse_evaluator_params(root_element)
        subato_id = self.parse_attribute(root_element, 'subatoId', int)
        attempts = self.parse_attribute(root_element, 'attempts', int)
        lang = self.parse_attribute(root_element, 'lang')
        files = self.parse_files(root_element)
        return Meta(
            subato_id=subato_id,
            name=name,
            description=description,
            evaluator=evaluator,
            evaluator_params=params,
            lang=lang,
            files=files,
            attempts=attempts,
        )

    def parse_element_text(self, root, element_name):
        ele = root.find(element_name)
        return ''.join(ele.itertext()) if ele is not None else None

    def parse_attribute(self, ele, attr, fn=None):
        v = ele.get(attr) if ele.get(attr) is not None else None
        return fn(v) if v is not None and fn is not None else v

    def parse_files(self, root):
        file_elements = root.findall('.//file')
        files = []
        for ele in file_elements:
            path = ele.get('path')
            file_type = ele.get('type')
            public = ele.get('public').lower() == 'true' if ele.get('public') is not None else False
            files.append(File(path=path, type=file_type, public=public))

        return files

    def parse_evaluator_params(self, root):
        param_elements = root.findall('.//param')
        params = {}
        for ele in param_elements:
            key = ele.get('key')
            val = ele.get('value')
            params[key] = val

        return params
