import os
from typing import List

from eva.playground.stef import StefParser, StefTask
from pydantic import BaseModel, ConfigDict


class TaskPool(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)
    id: str
    tasks: List[StefTask] = []

    def __eq__(self, other):
        return other is not None and self.id == other.id

    def __hash__(self):
        return hash(self.id)


class TaskManager:
    def __init__(self, dir_path):
        self.dir_path = dir_path
        self.pools = []

    def load_tasks(self):
        pool_dirs = [f.name for f in os.scandir(self.dir_path) if f.is_dir()]
        self.pools = []
        for pool_dir in pool_dirs:
            tasks = self._load_pool(pool_dir)
            if len(tasks) != 0:
                self.pools.append(TaskPool(id=pool_dir, tasks=tasks))

        return {pool: pool.tasks for pool in self.pools}

    def _load_pool(self, dir_name):
        pool_dir = os.path.join(self.dir_path, dir_name)
        tasks = []
        parser = StefParser()

        for root, dirs, _ in os.walk(pool_dir):
            for subdir in dirs:
                task_path = os.path.join(root, subdir)
                try:
                    task = parser.parse(task_path)
                    if task is not None and task.meta.evaluator is not None:
                        tasks.append(task)
                # pylint: disable=broad-exception-caught
                except Exception:
                    print('invalid task:', task_path)

        return tasks
