import contextlib
import sys
from io import StringIO


def flatten(l):
    return [item for sublist in l for item in sublist]


@contextlib.contextmanager
def capture_logs(log_level):
    try:
        # pylint: disable=import-outside-toplevel
        from loguru import logger

        logger.remove()
        args = {
            'format': '<green>{time}</green> {name} {level} <level>{message}</level>',
            'level': log_level,
            'backtrace': False,
            'diagnose': False,
        }

        logger.add(sys.stdout, colorize=True, **args)

        s = StringIO()

        def add_to_state(m, s):
            s.write(str(m))

        logger.add(lambda m: add_to_state(m, s), level=log_level)
        yield s
    finally:
        pass
