from eva.pltform.evaluation.c.util import main_function
from eva.pltform.evaluation.core import (
    CompilationResult,
    CompileHandler,
    EvaluationContext,
    Evaluator,
)
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


class CCompileHandler(CompileHandler):
    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> CompilationResult:
        if any(main_function(f.content) for f in ctx.payload.solution_files):
            return CompilationResult(
                compiling=False,
                output='In einer der Lösungsdateien befindet sich eine '
                'main-Funktion. Diese Funktion muss für die '
                'automatische Auswertung entfernt werden.',
            )

        r = ctx.env.bash(['find . -type f -name "*.c"'])
        r.expect(status=[CODE_SUCCESS])
        cfiles = r.stdout.split('\n')

        r = ctx.env.bash(
            [
                'gcc -ggdb -O2 -fPIE -pie -Wformat -Wformat-security -Wall -lm -o test-runner',
                *cfiles,
            ]
        )
        r.expect(status_not=[CODE_TIMEOUT])

        compilation_failed = r.retcode != 0
        return CompilationResult(compiling=not compilation_failed, output=r.out())
