from pathlib import Path

from eva.pltform.evaluation.core import EvalPayload


def generate_cmake(payload: EvalPayload):
    base = """
cmake_minimum_required(VERSION 3.14)
project(my_project)

# GoogleTest requires at least C++14
set(CMAKE_CXX_STANDARD 14)

include(FetchContent)
FetchContent_Declare(
  googletest
  URL file://$ENV{GOOGLETEST_PATH}
)
# For Windows: Prevent overriding the parent project's compiler/linker settings
set(gtest_force_shared_crt ON CACHE BOOL "" FORCE)
FetchContent_MakeAvailable(googletest)

enable_testing()
link_libraries(GTest::gtest_main)

include_directories(${CMAKE_CURRENT_SOURCE_DIR})
"""

    solution_paths = [f.path for f in payload.solution_files]
    include_paths = [f.path for f in payload.include_files]

    for t in payload.test_files:
        name = Path(t.path).stem
        args = [name] + [t.path] + solution_paths + include_paths + ['ads.cpp']
        base += f"add_executable({' '.join(args)})\n"

    base += 'include(GoogleTest)\n'

    for t in payload.test_files:
        name = Path(t.path).stem
        base += f'gtest_discover_tests({name})\n'

    return base
