import os

from eva.pltform.evaluation.c.util import main_function
from eva.pltform.evaluation.core import (
    CompilationResult,
    CompileHandler,
    EvaluationContext,
    Evaluator,
)
from eva.pltform.evaluation.cpp.cmake import generate_cmake
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


class CppCompileHandler(CompileHandler):
    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> CompilationResult:
        if any(main_function(f.content) for f in ctx.payload.solution_files):
            return CompilationResult(
                compiling=False,
                output='In einer der Lösungsdateien befindet sich eine '
                'main-Funktion. Diese Funktion muss für die '
                'automatische Auswertung entfernt werden.',
            )

        cmake = generate_cmake(ctx.payload)
        ctx.env.create_file(cmake, os.path.join(ctx.env.cwd, 'CMakeLists.txt'))

        r = ctx.env.bash(['cmake -S . -B build'])
        r.expect(status=[CODE_SUCCESS])

        r = ctx.env.bash(['cmake --build build'])
        r.expect(status_not=[CODE_TIMEOUT])

        compilation_failed = r.retcode != 0
        return CompilationResult(compiling=not compilation_failed, output=r.out())
