from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_TIMEOUT


class CppTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(
            [
                'cd build && GTEST_OUTPUT="xml:gtest_results.xml" ctest '
                '--output-junit gtest_results.xml'
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )

        if r.retcode == CODE_TIMEOUT:
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        testsuite = self.trex.parse(ctx.env.read_file('build/gtest_results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
