from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


class ElmTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        test_paths = [f.path for f in ctx.payload.test_files]
        other_paths = [f.path for f in ctx.payload.solution_files + ctx.payload.include_files]

        r = ctx.env.bash(['mv', *test_paths, 'tests/'])
        r.expect(status=[CODE_SUCCESS])

        r = ctx.env.bash(['mv', *other_paths, 'src/'])
        r.expect(status=[CODE_SUCCESS])

        r = ctx.env.bash(
            ['elm-test --report=junit > results.xml'],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )

        # syntax errors lead to an empty results.xml
        has_results = ctx.env.file_exists('results.xml') and ctx.env.read_file('results.xml')

        if r.retcode == CODE_TIMEOUT or not has_results:
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        testsuite = self.trex.parse(ctx.env.read_file('results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
