from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_TIMEOUT


class EvarelaTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        if len(ctx.payload.solution_files) > 1:
            raise RuntimeError('this evaluator requires exactly one solution file')

        if len(ctx.payload.test_files) > 1:
            raise RuntimeError('this evaluator requires exactly one test file')

        r = ctx.env.bash(
            [
                'TestEva',
                ctx.payload.solution_files[0].path,
                ctx.payload.test_files[0].path,
                '>',
                'results.xml',
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )

        if r.retcode == CODE_TIMEOUT:
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        testsuite = self.trex.parse(ctx.env.read_file('results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
