from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_TIMEOUT


class GoTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(
            ['./test_binary -test.v 2>&1 | go-junit-report > report.xml'],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )

        if r.retcode == CODE_TIMEOUT:
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        testsuite = self.trex.parse(ctx.env.read_file('report.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
