from eva.pltform.format.stex import StyleError, StyleReport
from lxml import etree
from lxml.etree import LxmlError


class CheckstyleParser:

    def parse(self, xml) -> StyleReport:
        xml = xml.strip().replace('encoding="UTF-8"', '')

        try:
            root = etree.fromstring(xml)
            version = root.attrib.get('version')

            errors = []
            for file_node in root.xpath('file'):
                for error_node in file_node.xpath('error'):
                    name = file_node.attrib.get('name')
                    line = int(error_node.attrib.get('line'))
                    column = int(error_node.attrib.get('column'))
                    severity = error_node.attrib.get('severity')
                    message = error_node.attrib.get('message')
                    source = error_node.attrib.get('source')

                    errors.append(
                        StyleError(
                            file_path=name,
                            line=line,
                            column=column,
                            severity=severity,
                            message=message,
                            type=source,
                        )
                    )

            return StyleReport(errors=errors, tool='checkstyle', version=version)
        except LxmlError as e:
            raise RuntimeError('checkstyle report could not be parsed') from e
