from eva.pltform.evaluation.core import (
    EvaluationContext,
    Evaluator,
    EvaluatorCapability,
    ExecutionHandlerConfig,
)
from eva.pltform.evaluation.java.compile import JavaCompileHandler
from eva.pltform.evaluation.java.jvm import configure_jvm
from eva.pltform.evaluation.java.style import create_style_handler
from eva.pltform.evaluation.java.test import create_test_handler
from eva.pltform.evaluation.java.validation import check_dynamic_asserts


class JavaEvaluator(Evaluator):
    id = 'java'
    supported_envs = 'java*'
    default_env = 'java19'
    capability = EvaluatorCapability(compile=True, style=True, test=True)

    def setup(self, ctx: EvaluationContext):
        check_dynamic_asserts(ctx)
        java_args, javac_args = configure_jvm(ctx)

        return ExecutionHandlerConfig(
            compile=JavaCompileHandler(javac_args=javac_args),
            style=create_style_handler(ctx, javac_args, java_args),
            test=create_test_handler(ctx, javac_args, java_args),
        )
