from eva.pltform.format.stex import StyleError, StyleReport
from lxml import etree
from lxml.etree import LxmlError


class SpotBugsParser:

    def parse(self, xml) -> StyleReport:
        xml = xml.strip().replace('encoding="UTF-8"', '')

        try:
            root = etree.fromstring(xml)
            version = '4.8.3'  # root.attrib.get('version')

            # <Errors errors="0" missingClasses="0"></Errors>
            errors_nodes = root.xpath('Errors')
            errors_node = errors_nodes[0] if len(errors_nodes) != 0 else None
            if errors_node is not None:
                error_count = int(errors_node.attrib.get('errors'))
                if error_count != 0:
                    raise RuntimeError(f'spotbugs reported {error_count} errors')

                missing_classes = int(errors_node.attrib.get('missingClasses'))
                if error_count != 0:
                    raise RuntimeError(f'spotbugs reported {missing_classes} missing classes')

            errors = []
            for file_node in root.xpath('BugInstance'):
                err_type = file_node.attrib.get('type')
                severity = file_node.attrib.get('category') or ''

                bug_xml = etree.tostring(file_node).decode('utf-8')

                for error_node in file_node.xpath('SourceLine'):
                    line = int(error_node.attrib.get('start'))
                    column = 0
                    message = ''
                    file_path = error_node.attrib.get('sourcepath')

                    err = StyleError(
                        file_path=file_path,
                        line=line,
                        column=column,
                        severity=severity,
                        message=message,
                        type=err_type,
                        data={'xml': bug_xml},
                    )
                    errors.append(err)

            return StyleReport(errors=errors, tool='spotbugs', version=version)
        except LxmlError as e:
            raise RuntimeError('spotbugs report could not be parsed') from e
