from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


class JsTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(
            ['mocha test --reporter mocha-xunit-reporter'],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )
        runtime_output = r.out()

        # inf loop or syntax error
        if r.retcode == CODE_TIMEOUT or not ctx.env.file_exists('test-results.xml'):
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        r = ctx.env.bash(
            ['xsltproc transform.xsl test-results.xml > results.xml'],
            max_output_size=ctx.opts.max_output_size,
        )
        r.expect(status=[CODE_SUCCESS])

        testsuite = self.trex.parse(ctx.env.read_file('results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=runtime_output)
