from eva.pltform.evaluation.core import (
    EvaFile,
    EvaluationContext,
    Evaluator,
    TestHandler,
    TestResult,
)
from eva.pltform.format.trex import TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


def to_fqcn(file: EvaFile):
    return file.path.replace('.kt', '').replace('/', '.')


class KotlinTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        test_fqcns = [to_fqcn(f) for f in ctx.payload.test_files]
        r = ctx.env.bash(
            [
                'kotlin -cp $EJUNIT_RUNNER_PATH:$CLASSPATH:classes',
                'de.hsrm.sls.ejunit.mono.Runner -o trex_report.xml',
                f'-s {ctx.opts.max_output_size}',
                *test_fqcns,
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )
        r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])

        testsuite = self.trex.parse(ctx.env.read_file('trex_report.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
