from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


class MariaDBTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(['init_db.sh'])
        r.expect(status=[CODE_SUCCESS])

        test_paths = [f.path for f in ctx.payload.test_files]
        solution_paths = [f.path for f in ctx.payload.solution_files]

        r = ctx.env.bash(
            [
                'java -cp $SQLUNIT_PATH de.hsrm.sls.sqlunit.Runner',
                f"-s {','.join(solution_paths)} -o results.xml",
                *test_paths,
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )
        r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])

        testsuite = self.trex.parse(ctx.env.read_file('results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
