from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_TIMEOUT


class PythonTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        test_paths = [f.path for f in ctx.payload.test_files]
        r = ctx.env.bash(
            ['python -m xmlrunner', *test_paths, '--output-file results.xml'],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )

        # inf loop or syntax error
        if r.retcode == CODE_TIMEOUT or not ctx.env.file_exists('results.xml'):
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        testsuite = self.trex.parse(ctx.env.read_file('results.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
