from eva.pltform.evaluation.core import (
    EvaFile,
    EvaluationContext,
    Evaluator,
    TestHandler,
    TestResult,
)
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_TIMEOUT


def to_fqcn(file: EvaFile):
    return file.path.replace('.scala', '').replace('/', '.')


class ScalaTestHandler(TestHandler):
    trex = TREXParser()

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(
            ['ant test'], max_output_size=ctx.opts.max_output_size, timeout=ctx.opts.test_timeout
        )

        if r.retcode == CODE_TIMEOUT:
            testsuite = Testsuite(testcases=[])
            return self.mapper.map(testsuite=testsuite, output=r.out())

        test_fqcn = to_fqcn(ctx.payload.test_files[0])
        testsuite = self.trex.parse(ctx.env.read_file(f'TEST-{test_fqcn}.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())
