import threading
from dataclasses import dataclass

import loguru
from eva.pltform.rt.core.exception import RuntimeInterruptedError
from eva.pltform.rt.core.result import ExecResult

CODE_TIMEOUT = 283
CODE_SUCCESS = 0
CODE_UNKNOWN_CMD = 127


@dataclass
class EnvironmentHandle:
    environment_id: str
    cwd: str
    stopped: threading.Event()
    request_logger: 'loguru.Logger'

    def ensure_alive(self):
        if self.stopped.is_set():
            raise RuntimeInterruptedError()

    def exec(self, cmd, timeout: int = None, max_output_size: int = None) -> ExecResult:
        raise NotImplementedError()

    def bash(self, cmd, **kwargs) -> ExecResult:
        cmd_raw = ' '.join(cmd).replace('"', '\\"')
        return self.exec([f'bash -c "{cmd_raw}"'], **kwargs)

    def file_exists(self, path: str) -> bool:
        r = self.exec([f'cat {path}'])
        return r.retcode != 1

    def read_file(self, path: str, **kwargs):
        """
        reads the file at :path and returns its content.
        :raises ExpectationFailedError: if the file does not exist.
        """
        r = self.exec([f'cat {path}'], **kwargs)
        r.expect(status=[CODE_SUCCESS])
        return r.stdout

    def mkdir(self, path: str, recursive=False, **kwargs):
        cmd = ['mkdir']
        if recursive:
            cmd += ['-p']

        cmd += [path]
        r = self.exec(cmd, **kwargs)
        r.expect(status=[CODE_SUCCESS])

    def create_file(self, content: str, path: str, **kwargs):
        """copies the string content as file contents to path in the docker container"""
        raise NotImplementedError()
