from dataclasses import dataclass


class ExpectationFailedError(Exception):
    def __init__(self, exec_result: 'ExecResult', reason: str):
        super().__init__(f'execution failed: {reason} ({exec_result.cmd})')


@dataclass
class ExecResult:
    cmd: str
    retcode: int
    stdout: str
    stderr: str

    def out(self):
        merged = self.stdout + self.stderr
        return merged

    def expect(self, status=None, status_not=None, stdout=None, stderr=None):
        if status is not None:
            if self.retcode not in status:
                raise ExpectationFailedError(self, f'status code {self.retcode} not in {status}')

        if status_not is not None:
            if self.retcode in status_not:
                raise ExpectationFailedError(self, f'status code {self.retcode} in {status_not}')

        if stdout is not None:
            if stdout and len(self.stdout) == 0:
                raise ExpectationFailedError(self, 'stdout empty')

            if not stdout and len(self.stdout) != 0:
                raise ExpectationFailedError(self, 'stdout not empty')

        if stderr is not None:
            if stderr and len(self.stderr) == 0:
                raise ExpectationFailedError(self, 'stderr empty')

            if not stderr and len(self.stderr) != 0:
                raise ExpectationFailedError(self, 'stderr not empty')
