from eva.pltform.format.trex import PendingResult, StartedResult, TREXParser
from tests.util import FixtureLoader


def test_stats(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))
    assert 6 == len(testrun.testcases)
    assert 2 == testrun.failures()
    assert 1 == testrun.errors()


def test_storage(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))

    assert testrun.free is None
    assert testrun.allocated is None
    assert testrun.leaks() is None


def test_testcase(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))
    testcase = testrun.testcases[0]

    assert 'de.hsrm.sls.eva.junit.samples.simple.PingTest#test01' == testcase.id
    assert 'test01' == testcase.name
    assert 'de.hsrm.sls.eva.junit.samples.simple.PingTest' == testcase.class_name


def test_failure(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))
    testcase = testrun.testcases[1]

    assert 'expected:<43> but was:<42>' == testcase.result.message
    # pylint: disable=line-too-long
    assert (
        '''java.lang.AssertionError: expected:<43> but was:<42>
    at de.hsrm.sls.eva.junit.samples.simple.PingTest.test02(PingTest.java:23)
    at java.base/jdk.internal.reflect.DirectMethodHandleAccessor.invoke(DirectMethodHandleAccessor.java:104)
    at de.hsrm.sls.eva.junit.Runner.runTests(Runner.java:28)
    at de.hsrm.sls.eva.junit.Runner.main(Runner.java:18)'''
        == testcase.result.details
    )


def test_pending(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex3.xml'))
    assert isinstance(testrun.testcases[1].result, PendingResult)


def test_started(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex3.xml'))
    assert isinstance(testrun.testcases[0].result, StartedResult)


def test_std1(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))

    assert testrun.stdout is None
    assert testrun.stderr is None


def test_std2(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex2.xml'))

    assert (
        '''JAA
MAGIC
MAGIC'''
        == testrun.stdout
    )
    assert '''BUUUUH''' == testrun.stderr


def test_testcase_std1(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex1.xml'))
    testcase = testrun.testcases[0]

    assert testcase.stdout is None
    assert testcase.stderr is None


def test_testcase_std2(fixtures: FixtureLoader):
    parser = TREXParser()
    testrun = parser.parse(fixtures.load('trex2.xml'))
    testcase7 = testrun.testcases[6]

    assert 'JAA' == testcase7.stdout
    assert 'BUUUUH' == testcase7.stderr
