from html import escape

from eva.pltform.evaluation.core import EvaFile
from eva.pltform.format.cexf import Coverage, LineDecision, LineType
from IPython.core.display import HTML
from IPython.core.display_functions import display


def render_coverage(solution_file, result, testcase):
    highlights = highlight_coverage(solution_file, result.test.coverage)
    html = highlights[testcase]
    header = f'{solution_file.path} - {testcase} ({result.test.coverage.tool})'
    print(header)
    print('=' * len(header))
    display(HTML(html))


def highlight_coverage(file: EvaFile, coverage: Coverage):
    highlights = {}
    for testcase in coverage.testsuite.testcases:
        file_coverage = next(iter(f for f in testcase.files if f.name == file.path), None)
        if file_coverage is None:
            continue

        file_lines = file.content.split('\n')
        for i, line in enumerate(file_lines):
            l = next(
                iter([l for l in file_coverage.lines if l.type == LineType.LINE and l.nr - 1 == i]),
                None,
            )

            color = 'transparent'
            if l is not None:
                if l.decision == LineDecision.FULL:
                    color = '#b8ffb8'
                elif l.decision == LineDecision.PARTIAL:
                    color = '#ffff9a'
                elif l.decision == LineDecision.MISSED:
                    color = '#ffb3b3'

            new_line = (
                f'<div style="background-color: {color}; ">{str(i + 1)} ' f'{escape(line)}</div>'
            )
            file_lines[i] = new_line

        html_code = (
            ('<pre style="font-size: 12px; background-color: white;padding: 10px;color: ' 'black">')
            + ''.join(file_lines)
            + '</pre>'
        )
        highlights[testcase.id] = html_code

    return highlights
