import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('hs')
class HaskellTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='Fac.hs', content=ctx.fixtures.load('hs/passing/Fac.hs'))],
            include_files=[
                EvaFile(
                    path='TestsPublic.hs', content=ctx.fixtures.load('hs/passing/TestsPublic.hs')
                )
            ],
            test_files=[
                EvaFile(path='MyTest.hs', content=ctx.fixtures.load('hs/passing/MyTest.hs')),
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.hs', content=ctx.fixtures.load('hs/compfailed/Fac.hs'))
            ],
            include_files=[
                EvaFile(
                    path='TestsPublic.hs', content=ctx.fixtures.load('hs/compfailed/TestsPublic.hs')
                )
            ],
            test_files=[
                EvaFile(path='MyTest.hs', content=ctx.fixtures.load('hs/compfailed/MyTest.hs')),
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert "error: parse error on input `=>'" in result.compilation.output

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='Fac.hs', content=ctx.fixtures.load('hs/failure/Fac.hs'))],
            include_files=[
                EvaFile(
                    path='TestsPublic.hs', content=ctx.fixtures.load('hs/failure/TestsPublic.hs')
                )
            ],
            test_files=[
                EvaFile(path='MyTest.hs', content=ctx.fixtures.load('hs/failure/MyTest.hs')),
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.hs', content=ctx.fixtures.load('hs/stackoverflow/Fac.hs'))
            ],
            include_files=[
                EvaFile(
                    path='TestsPublic.hs',
                    content=ctx.fixtures.load('hs/stackoverflow/TestsPublic.hs'),
                )
            ],
            test_files=[
                EvaFile(path='MyTest.hs', content=ctx.fixtures.load('hs/stackoverflow/MyTest.hs')),
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 00
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
