import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('java')
class JavaOutputsTest:

    def test_outputs(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/outputs/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/outputs/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert result.test.output
        assert (
            result.test.testsuite.stdout
            == """sqrt: 9.0
sqrt: 10000.0
sqrt: 2.0"""
        )
        assert result.test.testsuite.stderr == """err: 9.0"""
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testsuite.testcases[0].stdout == 'sqrt: 9.0'
        assert result.test.testsuite.testcases[0].stderr == 'err: 9.0'

        assert result.test.testsuite.testcases[1].stdout == 'sqrt: 10000.0'
        assert result.test.testsuite.testcases[1].stderr == ''

        assert result.test.testsuite.testcases[2].stdout == 'sqrt: 2.0'
        assert result.test.testsuite.testcases[2].stderr == ''

    def test_memoverflow_output(self, ctx: TestContext):
        # if the solution prints several dozens of megabytes to stdout, the system should
        # truncate the outputs
        # instead of failing due to out of memory or buffer overflow issues

        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='AL.java', content=ctx.fixtures.load('java/memoverflow_output/AL.java')
                )
            ],
            test_files=[
                EvaFile(
                    path='ALTest.java',
                    content=ctx.fixtures.load('java/memoverflow_output/ALTest.java'),
                )
            ],
            include_files=[],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 1
        assert result.test.testcases == 1
        assert result.test.failed == 0
        assert result.test.passed == 1
        assert '(truncated)' in result.test.output
        assert 'truncated' in result.test.testsuite.stdout
        assert 'truncated' in result.test.testsuite.testcases[0].stdout
