import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('javaie')
class JavaIEOutputsTest:

    def test_outputs(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/outputs/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/outputs/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert result.test.output
        assert (
            result.test.testsuite.stdout
            == """sqrt: 9.0
sqrt: 10000.0
sqrt: 2.0"""
        )
        assert result.test.testsuite.stderr == """err: 9.0"""
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testsuite.testcases[0].stdout == 'sqrt: 9.0'
        assert result.test.testsuite.testcases[0].stderr == 'err: 9.0'

        assert result.test.testsuite.testcases[1].stdout == 'sqrt: 10000.0'
        assert result.test.testsuite.testcases[1].stderr == ''

        assert result.test.testsuite.testcases[2].stdout == 'sqrt: 2.0'
        assert result.test.testsuite.testcases[2].stderr == ''
