import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('js')
class JavaScriptTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.js', content=ctx.fixtures.load('js/passing/fac.js'))],
            test_files=[EvaFile(path='test.js', content=ctx.fixtures.load('js/passing/test.js'))],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_syntaxerr(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.js', content=ctx.fixtures.load('js/syntaxerr/fac.js'))
            ],
            test_files=[EvaFile(path='test.js', content=ctx.fixtures.load('js/syntaxerr/test.js'))],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert "SyntaxError: Unexpected token 'if'" in result.test.output
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.js', content=ctx.fixtures.load('js/failure/fac.js'))],
            test_files=[EvaFile(path='test.js', content=ctx.fixtures.load('js/failure/test.js'))],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.js', content=ctx.fixtures.load('js/infloop/fac.js'))],
            test_files=[EvaFile(path='test.js', content=ctx.fixtures.load('js/infloop/test.js'))],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.js', content=ctx.fixtures.load('js/stackoverflow/fac.js'))
            ],
            test_files=[
                EvaFile(path='test.js', content=ctx.fixtures.load('js/stackoverflow/test.js'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)
