import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import ErrorResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('scala')
class ScalaTest:
    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.scala', content=ctx.fixtures.load('scala/passing/Fac.scala'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.scala', content=ctx.fixtures.load('scala/passing/FacTest.scala')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.scala', content=ctx.fixtures.load('scala/compfailed/Fac.scala'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.scala',
                    content=ctx.fixtures.load('scala/compfailed/FacTest.scala'),
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert "error: Unmatched closing brace '}' ignored here" in result.compilation.output

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.scala', content=ctx.fixtures.load('scala/failure/Fac.scala'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.scala', content=ctx.fixtures.load('scala/failure/FacTest.scala')
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 3
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, ErrorResult)

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='Fac.scala', content=ctx.fixtures.load('scala/stackoverflow/Fac.scala')
                )
            ],
            test_files=[
                EvaFile(
                    path='FacTest.scala',
                    content=ctx.fixtures.load('scala/stackoverflow/FacTest.scala'),
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 3
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, ErrorResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.scala', content=ctx.fixtures.load('scala/infloop/Fac.scala'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.scala', content=ctx.fixtures.load('scala/infloop/FacTest.scala')
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
