import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('turing')
class TuringTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.txt', content=ctx.fixtures.load('turing/passing/solution.txt')
                )
            ],
            test_files=[
                EvaFile(
                    path='simpleTest.turing',
                    content=ctx.fixtures.load('turing/passing/simpleTest.turing'),
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 5
        assert result.test.testcases == 5
        assert result.test.failed == 0
        assert result.test.passed == 5
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_syntaxerr(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.txt', content=ctx.fixtures.load('turing/syntaxerr/solution.txt')
                )
            ],
            test_files=[
                EvaFile(
                    path='simpleTest.turing',
                    content=ctx.fixtures.load('turing/syntaxerr/simpleTest.turing'),
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 1
        assert result.test.testcases == 1
        assert result.test.failed == 1
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert result.test.testsuite.testcases[0].id == 'Kompilierung'

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.txt', content=ctx.fixtures.load('turing/failure/solution.txt')
                )
            ],
            test_files=[
                EvaFile(
                    path='simpleTest.turing',
                    content=ctx.fixtures.load('turing/failure/simpleTest.turing'),
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 5
        assert result.test.testcases == 5
        assert result.test.failed == 5
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 5
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)
