import datetime
from typing import List

import docker
from eva.registry.environ.cache import EnvironmentsCache
from eva.registry.environ.model import Environment
from eva.registry.environ.provider import EnvironmentProvider


class LocalDockerEnvironmentProvider(EnvironmentProvider):
    def __init__(self, cache: EnvironmentsCache, img_base_path: str):
        self.img_base_path = img_base_path
        self.cache = cache

    def get_envs(self) -> (List[Environment], datetime.datetime):
        if not self.cache.expired():
            return self.cache.get(), self.cache.last_update

        client = docker.from_env()

        environments = []
        for image in client.images.list():
            for img in image.tags:
                if self.img_base_path in img:
                    image_name, tag = img.split('/')[-1].rsplit(':', 1)
                    if tag == 'latest':
                        labels = image.labels

                        description = labels.get('DESCRIPTION', 'undefined')
                        maintainer = labels.get('MAINTAINER', 'undefined')

                        evaluator = Environment(
                            id=image_name, description=description, maintainer=maintainer, tag=tag
                        )
                        environments.append(evaluator)

        self.cache.set(environments)
        return environments, self.cache.last_update
