import datetime
import fnmatch
from typing import List

from eva.pltform import EvaluatorRegistry
from eva.registry.environ import Environment, EnvironmentProvider
from eva.registry.stack.model import EvaluationStack, EvaluatorDeclaration


def _supports_env(evaluator: EvaluatorDeclaration, env: Environment):
    if evaluator.supported_envs is not None:
        supported = fnmatch.fnmatch(env.id, evaluator.supported_envs)
        if supported and evaluator.excluded_envs is not None:
            should_exclude = fnmatch.fnmatch(env.id, evaluator.excluded_envs)
            return not should_exclude

        return supported

    return False


class EvaluationStackProvider:
    def __init__(self, env_provider: EnvironmentProvider):
        self.env_provider = env_provider
        self.registry = EvaluatorRegistry()

    def _get_evaluators(self) -> List[EvaluatorDeclaration]:
        return [
            EvaluatorDeclaration(
                id=clazz.id,
                capability=clazz.capability,
                supported_envs=clazz.supported_envs,
                excluded_envs=clazz.excluded_envs,
                default_env=clazz.default_env,
            )
            for clazz in self.registry.get_evaluators()
        ]

    def get_stacks(self) -> (List[EvaluationStack], datetime.datetime):
        environments, last_update = self.env_provider.get_envs()
        evaluators = self._get_evaluators()

        stacks = []
        for evaluator in evaluators:
            for env in environments:
                if _supports_env(evaluator, env):
                    stacks.append(EvaluationStack(evaluator=evaluator, environment=env))

        return stacks, last_update
