import re

from eva.registry.stack.provider import EvaluationStackProvider


class EvaluationStackNotFoundError(Exception):
    pass


class EvaluationStackQuery:
    def __init__(self, ref: str):
        self.ref = ref

    def find(self, stacks_provider: EvaluationStackProvider):
        match = re.search(r'([^@]+)(?:@(\S+))?', self.ref)
        if match is None:
            raise EvaluationStackNotFoundError(f'no stack found for ref {self.ref}')

        evaluator_id = match.group(1)
        environment_id = match.group(2)

        stacks, last_update = stacks_provider.get_stacks()
        if environment_id is not None:
            stack = next(
                iter(
                    [
                        s
                        for s in stacks
                        if s.evaluator.id == evaluator_id and s.environment.id == environment_id
                    ]
                ),
                None,
            )
        else:
            stack = next(
                iter(
                    [
                        s
                        for s in stacks
                        if s.evaluator.id == evaluator_id
                        and s.environment.id == s.evaluator.default_env
                    ]
                ),
                None,
            )

        if stack is None:
            raise EvaluationStackNotFoundError(f'no stack found for ref {self.ref}')

        return stack, last_update
