package de.hsrm.sls.cexf.jacoco;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class JacocoCoverageLoader {

    private final String directoryPath;

    public JacocoCoverageLoader(String directoryPath) {
        this.directoryPath = directoryPath;
    }

    public Map<String, byte[]> loadExecFiles() {
        var execFilesData = new HashMap<String, byte[]>();

        File directory = new File(directoryPath);
        if (!directory.exists() || !directory.isDirectory()) {
            throw new RuntimeException("Invalid directory path: " + directoryPath);
        }

        File[] files = directory.listFiles(
            (dir, name) -> name.endsWith(".exec") && !name.endsWith("jacoco.exec"));
        if (files == null) {
            throw new RuntimeException("Error listing files in the directory:" + directoryPath);
        }

        for (File file : files) {
            try {
                byte[] fileData = readFileAsByteArray(file);
                execFilesData.put(getFileNameWithoutExtension(file), fileData);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        return execFilesData;
    }

    private static String getFileNameWithoutExtension(File file) {
        String fileName = file.getName();

        int lastDotIndex = fileName.lastIndexOf('.');
        if (lastDotIndex > 0) {
            return fileName.substring(0, lastDotIndex);
        }

        // If there is no extension or the filename starts with a dot (hidden file),
        // then return the full filename as it is.
        return fileName;
    }

    private static byte[] readFileAsByteArray(File file) throws IOException {
        try (var fis = new FileInputStream(file)) {
            var data = new byte[(int) file.length()];
            fis.read(data);
            return data;
        }
    }
}
