package de.hsrm.sls.cexf.openclover;

import com.atlassian.clover.api.command.ArgProcessor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.stream.Collectors;

public class ArgProcessors {
    public static ArgProcessor<CEXFConfig> FileFilter = new ArgProcessor<>() {
        public boolean matches(String[] args, int i) {
            return args[i].equals("-ff") || args[i].equals("--file");
        }

        public int process(String[] args, int i, CEXFConfig cfg) {
            ++i;

            var raw = args[i];
            var filter = raw != null && !raw.isEmpty()
                ? Arrays.stream(raw.split(",")).toList()
                : new ArrayList<String>();
            cfg.setFiles(filter);

            return i;
        }

        public String help() {
            return "    -ff,  --file [fqcn1 [,fqcn2 [,fqcn3] ...]]\tComma separated list of FQCNs for filtering code coverage.\n\t\t\t\tIn contrast to the JaCoCo report generator, it is NOT\n\t\t\t\trequired that the classes are on the class path.\n\n";
        }
    };

    public static ArgProcessor<CEXFConfig> LineTypeFilter = new ArgProcessor<>() {
        public boolean matches(String[] args, int i) {
            return args[i].equals("-lt") || args[i].equals("--ltype");
        }

        public int process(String[] args, int i, CEXFConfig cfg) {
            ++i;
            var identifier = args[i];
            var lineType = Arrays.stream(CEXFLineType.values()).filter(lt -> lt.getIdentifier().equals(identifier)).findFirst().orElse(null);
            cfg.setLineType(lineType);
            return i;
        }

        public String help() {
            var lineTypes = Arrays.stream(CEXFLineType.values()).map(CEXFLineType::getIdentifier).collect(
                Collectors.joining(", "));
            return "    -lt,  --ltype <string>\tCode coverage granularity to include in the report.\n\t\t\t\tOne of the following: %s.\n\t\t\t\tIf not specified, all types will be included in the report.\n\n".formatted(lineTypes);
        }
    };
}
