#!/bin/bash

set -e

# Function to display usage
usage() {
    echo "Usage: $0 -ff FILTERCLASS [class1 [class2 [class3] ...]]"
    exit 1
}

FILTERCLASS=""
TESTCLASSES=()

# Parse command line arguments
while [ "$#" -gt 0 ]; do
    case "$1" in
        -ff)
            shift
            FILTERCLASS="$1"
            shift
            ;;
        *)
            TESTCLASSES+=("$1")
            shift
            ;;
    esac
done

if [ -z "$FILTERCLASS" ]; then
    usage
fi

EJUNIT_ROOT=../..
LIBS_ROOT=$EJUNIT_ROOT/build/libs
CLOVER=$LIBS_ROOT/ejunit-openclover.jar
RUNNER=$LIBS_ROOT/ejunit-mono.jar

INITSTR=build/db/clover.db
CLASSES=build/classes
INSTRDIR=build/instr/
REPORT=build/report
SRC=$EJUNIT_ROOT/samples/src/main/java

$EJUNIT_ROOT/gradlew jar -p $EJUNIT_ROOT

rm -rf build

java -cp $CLOVER com.atlassian.clover.CloverInstr -i $INITSTR `find $SRC -type f -name "*.java"` -d $INSTRDIR --flushpolicy threaded --flushinterval 2000

mkdir -p $CLASSES
javac -classpath $RUNNER:$CLOVER -d $CLASSES `find $INSTRDIR -type f -name "*.java"`

java -cp $RUNNER:$CLOVER:$CLASSES de.hsrm.sls.ejunit.mono.Runner -o build/trex_report.xml ${TESTCLASSES[@]}

java -cp $CLOVER de.hsrm.sls.cexf.openclover.CEXFReport -ff $FILTERCLASS -i $INITSTR -o $REPORT/cexf_report.xml
java -cp $CLOVER com.atlassian.clover.reporters.html.HtmlReporter -if -i $INITSTR -o $REPORT
java -cp $CLOVER com.atlassian.clover.reporters.xml.XMLReporter   -i $INITSTR -o $REPORT/clover.xml
java -cp $CLOVER com.atlassian.clover.reporters.pdf.PDFReporter   -i $INITSTR -o $REPORT/clover.pdf
# java -cp $CLOVER com.atlassian.clover.reporters.console.ConsoleReporter -i $INITSTR -t "Sample Report"
cat $REPORT/cexf_report.xml