package de.hsrm.sls.ejunit.common;


import org.junit.runner.Description;
import org.junit.runner.notification.Failure;

public class JUnitUtils {

    public static boolean isAssertionFailure(Failure failure) {
        return failure.getException() instanceof AssertionError;
    }

    public static String getTestCaseName(Description d) {
        var s = d.getDisplayName();

        if (s == null || s.isEmpty()) {
            return null;
        }

        if (s.endsWith(")")) {
            int paren = s.lastIndexOf('(');
            return s.substring(0, paren);
        } else {
            return null;
        }
    }

    public static String getTestCaseClassName(Description d) {
        var s = d.getDisplayName();

        if (s == null || s.isEmpty()) {
            return null;
        }

        // JUnit 4 wraps solo tests this way. We can extract
        // the original test name with a little hack.
        int paren = s.lastIndexOf('(');
        if (paren != -1 && s.endsWith(")")) {
            return s.substring(paren + 1, s.length() - 1);
        } else {
            return s;
        }
    }

    /**
     * These errors occur if an exception was thrown in the RunListener. Just like initialization
     * errors, these are represented as standalone testcases.
     */
    public static boolean isTestMechanismError(Description descr) {
        return descr.getDisplayName().equals("Test mechanism");
    }

    /**
     * Surprisingly, JUnit does not throw exceptions on initialization errors. Instead, it adds the
     * error as testcase and includes the exception in the failure info.
     * <p>
     * Note that empty test classes will also cause an initialization error.
     */
    public static boolean isInitializationError(Description descr) {
        return descr.getMethodName() != null && descr.getMethodName().equals("initializationError");
    }

    public static boolean isJUnitErrorTestcase(Description descr) {
        return isTestMechanismError(descr) || isInitializationError(descr);
    }

}
