package de.hsrm.sls.ejunit.common;

import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

public class StackTraceFilter {

    private final String[] DEFAULT_TRACE_FILTERS = new String[]{"junit.framework.TestCase",
        "junit.framework.TestResult", "junit.framework.TestSuite", "junit.framework.Assert.",
        // don't filter AssertionFailure
        "junit.swingui.TestRunner", "junit.awtui.TestRunner", "junit.textui.TestRunner",
        "java.lang.reflect.Method.invoke(", "sun.reflect.", "org.apache.tools.ant.",
        // JUnit 4 support:
        "org.junit.", "junit.framework.JUnit4TestAdapter",
        // See wrapListener for reason:
        "Caused by: java.lang.AssertionError", " more",};


    /**
     * Returns a filtered stack trace. This is ripped out of junit.runner.BaseTestRunner.
     *
     * @param t the exception to filter.
     * @return the filtered stack trace.
     */
    public String applyFilter(Throwable t) {
        return filterStack(getStackTrace(t));
    }

    private boolean filterLine(String line) {
        for (int i = 0; i < DEFAULT_TRACE_FILTERS.length; i++) {
            if (line.indexOf(DEFAULT_TRACE_FILTERS[i]) != -1) {
                return true;
            }
        }
        return false;
    }

    /**
     * Filters stack frames from internal JUnit and Ant classes
     *
     * @param stack the stack trace to filter.
     * @return the filtered stack.
     */
    private String filterStack(String stack) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        StringReader sr = new StringReader(stack);
        BufferedReader br = new BufferedReader(sr);

        String line;
        try {
            while ((line = br.readLine()) != null) {
                if (!filterLine(line)) {
                    pw.println(line);
                }
            }
        } catch (Exception e) {
            return stack; // return the stack unfiltered
        }
        return sw.toString();
    }

    /**
     * Convenient method to retrieve the full stacktrace from a given exception.
     *
     * @param t the exception to get the stacktrace from.
     * @return the stacktrace from the given exception.
     */
    private String getStackTrace(Throwable t) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw, true);
        t.printStackTrace(pw);
        pw.flush();
        pw.close();
        return sw.toString();
    }

}
