package de.hsrm.sls.ejunit.common;

import org.junit.runner.Description;

public record TestcaseIdentifier(String className, String methodName) {

    public static TestcaseIdentifier fromString(String str) {
        var splitted = str.split("#");
        if (splitted.length != 2) {
            throw new RuntimeException("%s is not a valid testcase identifier".formatted(str));
        }

        return new TestcaseIdentifier(splitted[0], splitted[1]);
    }

    public static TestcaseIdentifier fromJUnit(Description test) {
        var methodName = JUnitUtils.getTestCaseName(test);
        if(methodName == null) {
            throw new RuntimeException("could not parse testcase name in JUnit description");
        }

        var className = JUnitUtils.getTestCaseClassName(test);
        if(className == null) {
            throw new RuntimeException("could not parse test class in JUnit description");
        }

        return new TestcaseIdentifier(className, methodName);
    }

    public String toString() {
        return "%s#%s".formatted(className, methodName);
    }
}
