package de.hsrm.sls.ejunit.ie.master;

import de.hsrm.sls.ejunit.common.Test;
import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import de.hsrm.sls.trex.Testcase;
import de.hsrm.sls.trex.Testsuite;
import de.hsrm.sls.trex.TimeoutResult;
import java.time.LocalDateTime;
import java.util.List;

public class DistributedTestRunner {

    private WorkerClient workerClient;

    public DistributedTestRunner(WorkerClient workerClient) {
        this.workerClient = workerClient;
    }

    public Testsuite run(List<Test> tests) {
        var testsuite = new Testsuite();
        testsuite.setStartedAt(LocalDateTime.now());

        for (var testclass : tests) {

            for (var method : testclass.getMethods()) {
                var testcaseId = new TestcaseIdentifier(testclass.getClazz().getName(), method);
                var testcase = workerClient.runTest(testcaseId);

                if (testcase == null) {
                    testcase = new Testcase();
                    testcase.setClassName(testclass.getClazz().getName());
                    testcase.setName(method);
                    testcase.setResult(new TimeoutResult(workerClient.getTimeout()));
                    testsuite.getTestcases().add(testcase);
                } else {
                    testsuite.getTestcases().add(testcase);
                }
            }
        }

        testsuite.setFinishedAt(LocalDateTime.now());
        return testsuite;
    }
}
