package de.hsrm.sls.ejunit.ie.master;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class RunArgs {

    private String workerPath;
    private String jacocoAgentPath;
    private String outPath;
    private int timeout;
    private String[] testclasses;

    public String getWorkerPath() {
        return workerPath;
    }

    public void setWorkerPath(String workerPath) {
        this.workerPath = workerPath;
    }

    public String getJacocoAgentPath() {
        return jacocoAgentPath;
    }

    public void setJacocoAgentPath(String jacocoAgentPath) {
        this.jacocoAgentPath = jacocoAgentPath;
    }

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }


    public String[] getTestclasses() {
        return testclasses;
    }

    public void setTestclasses(String[] testclasses) {
        this.testclasses = testclasses;
    }

    private static final String OPT_WORKER = "worker";
    private static final String OPT_TIMEOUT = "timeout";
    private static final String OPT_JACOCO_AGENT = "jacoco-agent";
    private static final String OPT_OUT_PATH = "out";

    private static CommandLine parseArgs(String[] args) {
        Options options = new Options();

        var wp = new Option("w", OPT_WORKER, true, "Path to the IE worker jar file.");
        wp.setRequired(true);
        options.addOption(wp);

        var timeout = new Option("t", OPT_TIMEOUT, true, "Execution timeout (per testcase) in seconds.");
        timeout.setRequired(false);
        options.addOption(timeout);

        var jacocoAgentPath = new Option("agent", OPT_JACOCO_AGENT, true,
            "Path to the jacocoagent jar file for enabling per-test coverage instrumentation with JaCoCo (online). Per-test coverage data will be saved in the working directory after each test in the following format: <testclass>#<testmethod>.exec");
        jacocoAgentPath.setRequired(false);
        options.addOption(jacocoAgentPath);

        var trexOutPath = new Option("o", OPT_OUT_PATH, true,
            "Output path for the resulting TREX report.");
        trexOutPath.setRequired(false);
        options.addOption(trexOutPath);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp(
                "ejunit-ie-master [OPTIONS] [class1 [class2 [class3] ...]]",
                "Executes all JUnit test methods in the given test classes identified by their FQCNs in SEPARATE JVMs. Test classes need to be on the class path with the code under test.\n\nOPTIONS:",
                options,
                "\nIf all test methods in the given test classes have finished, exit with status 0. This is independent from the result of the test case, meaning that even if test failures or errors occur, the status is 0. If an unexpected exception occurs in the test runner itself, return status 1.\n\nIn contrast to the mono runner, test methods are executed in their own JVM. Test methods are executed sequentially (not in parellel). For each test method, a JVM processes is started and waited for a result. If the JVM does not terminate after the configured timeout, the JVM process is killed. This will ensure that test methods running indefinitely will not prevent other test methods from being executed. Thus, this process may run for a total time = timeout * number of test methods in the test classes.\n\nThere is a significant overhead due to the start of a separate JVM for each test method, which slows down test execution in comparison to the mono runner.\n\nJVM processes are started with the -XX:MaxRAMPercentage=50 flag (to enable comparison with the mono runner) and inherit some flags from the spawning JVM (-ea, --enable-preview).");
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var workerPath = cmd.getOptionValue(OPT_WORKER);
        var jacocoAgentPath = cmd.getOptionValue(OPT_JACOCO_AGENT);
        var outPath = cmd.getOptionValue(OPT_OUT_PATH, "trex_report.xml");
        var timeout = Integer.parseInt(cmd.getOptionValue(OPT_TIMEOUT, "10"));

        var params = new RunArgs();
        params.setWorkerPath(workerPath);
        params.setJacocoAgentPath(jacocoAgentPath);
        params.setOutPath(outPath);
        params.setTimeout(timeout);
        params.setTestclasses(cmd.getArgs());
        return params;
    }
}
