package de.hsrm.sls.ejunit.ie.worker;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class RunArgs {

    private String outPath;
    private String testclass;
    private String method;
    private int maxOutputSize;

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    public String getTestclass() {
        return testclass;
    }

    public void setTestclass(String testclass) {
        this.testclass = testclass;
    }

    public String getMethod() {
        return method;
    }

    public void setMethod(String method) {
        this.method = method;
    }

    public int getMaxOutputSize() {
        return maxOutputSize;
    }

    public void setMaxOutputSize(int maxOutputSize) {
        this.maxOutputSize = maxOutputSize;
    }

    private static final String OPT_OUT_PATH = "out";
    private static final String OPT_METHOD = "method";
    private static final String OPT_CLASS = "clazz";
    private static final String OPT_MAX_OUTPUT_SIZE = "max-output-size";

    public static final int DEFAULT_MAX_OUTPUT_SIZE = 4096;

    private static CommandLine parseArgs(String[] args) {
        Options options = new Options();

        var outPath = new Option(OPT_OUT_PATH, true, "Output path for writing the test results.");
        outPath.setRequired(true);
        options.addOption(outPath);

        var method = new Option(OPT_METHOD, true,
            "Testcase to execute (name of the method in the test class).");
        method.setRequired(true);
        options.addOption(method);

        var testclass = new Option(OPT_CLASS, true,
            "FQCN of the test class. The class needs to exist in the class path.");
        testclass.setRequired(true);
        options.addOption(testclass);

        var maxOutputSize = new Option("s", OPT_MAX_OUTPUT_SIZE, true,
            "Maximum number of characters to include in stdout/stderr outputs from testcases before truncation. If the output is larger, only the last ARG characters are kept and a truncation notice is added to the output. Defaults to %s.".formatted(DEFAULT_MAX_OUTPUT_SIZE));
        maxOutputSize.setRequired(false);
        options.addOption(maxOutputSize);


        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp("ejunit-ie-worker [OPTIONS]\nOPTIONS:", options);
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var outPath = cmd.getOptionValue(OPT_OUT_PATH);
        var method = cmd.getOptionValue(OPT_METHOD);
        var testclass = cmd.getOptionValue(OPT_CLASS);
        var maxOutputSize = Integer.parseInt(cmd.getOptionValue(OPT_MAX_OUTPUT_SIZE, String.valueOf(DEFAULT_MAX_OUTPUT_SIZE)));

        var params = new RunArgs();
        params.setOutPath(outPath);
        params.setMethod(method);
        params.setTestclass(testclass);
        params.setMaxOutputSize(maxOutputSize);
        return params;
    }
}
