package de.hsrm.sls.ejunit.ls;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class RunArgs {

    private String outPath;
    private String[] testclasses;

    public String[] getTestclasses() {
        return testclasses;
    }

    public void setTestclasses(String[] testclasses) {
        this.testclasses = testclasses;
    }

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    private static final String OPT_OUT_PATH = "out";

    private static CommandLine parseArgs(String[] args) {
        Options options = new Options();

        var outPath = new Option("o", OPT_OUT_PATH, true,
            "Output path for writing the testcase identifiers. Testcase identifiers are written in the format <testclass>#<testmethod> into a text file with one identifier per line.");
        outPath.setRequired(false);
        options.addOption(outPath);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp(
                "ejunit-ls [OPTIONS] [class1 [class2 [class3] ...]]",
                "Extracts all test methods from the given test classes as testcase identifiers, that can be used as input for the mono runner. Test classes need to be on the class path with the code under test.\n\nOPTIONS:",
                options,
                "");
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var trexOutPath = cmd.getOptionValue(OPT_OUT_PATH, "identifiers.txt");

        var params = new RunArgs();
        params.setOutPath(trexOutPath);
        params.setTestclasses(cmd.getArgs());
        return params;
    }
}
