package de.hsrm.sls.ejunit.mono;

import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import java.io.File;
import java.io.FileOutputStream;
import org.jacoco.agent.rt.IAgent;
import org.jacoco.agent.rt.RT;
import org.junit.runner.Description;
import org.junit.runner.notification.RunListener;

/**
 * JUnit listener for recording coverage data using online bytecode instrumentation with the JaCoCo
 * agent. This implementation does not support concurrent execution of testcases and will write
 * coverage data only **at the end of a JUnit test**. If the JVM is killed or terminates abnormally,
 * the coverage data is lost.
 */
public class JacocoListener extends RunListener {

    private IAgent agent;

    public JacocoListener() {
        agent = RT.getAgent();
    }

    @Override
    public void testStarted(Description description) throws Exception {
        // Start the JaCoCo agent for the current test method
        agent.reset();
        var testcaseIdentifier = TestcaseIdentifier.fromJUnit(description);
        agent.setSessionId(testcaseIdentifier.toString());
    }

    @Override
    public void testFinished(Description description) throws Exception {
        var testcaseIdentifier = TestcaseIdentifier.fromJUnit(description);
        if (!agent.getSessionId().equals(testcaseIdentifier.toString())) {
            throw new RuntimeException(
                "Recording jacoco coverage does not support concurrent execution of testcases.");
        }

        // Write the execution data file for the current test method
        var data = agent.getExecutionData(true);
        var execFile = new File("%s.exec".formatted(testcaseIdentifier.toString()));
        try (var outputStream = new FileOutputStream(execFile)) {
            outputStream.write(data);
        }
    }
}