package de.hsrm.sls.ejunit.mono;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.List;

/**
 * JUnit handles exceptions thrown in listeners by creating failure objects. We want to prevent
 * failing silently, so we throw this exception manually.
 */
public class RunListenerException extends RuntimeException {

    private List<Throwable> exceptions;

    public RunListenerException(List<Throwable> exceptions) {
        super("%s exceptions occured while executing runner:\n\n%s".formatted(exceptions.size(),
            formatExceptions(exceptions)));
        this.exceptions = exceptions;
    }

    private static String formatExceptions(List<Throwable> exceptions) {
        var sb = new StringBuilder();
        for (var ex : exceptions) {
            sb.append("\t");

            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            ex.printStackTrace(new PrintStream(outputStream));
            sb.append(outputStream.toString());

            sb.append("\n");
        }

        return sb.toString();
    }

    public List<Throwable> getExceptions() {
        return exceptions;
    }
}
