package de.hsrm.sls.ejunit.mono;

import de.hsrm.sls.ejunit.common.JUnitUtils;
import de.hsrm.sls.ejunit.common.UncaughtExceptionHandler;
import de.hsrm.sls.ejunit.mono.filter.ExecFilterBuilder;
import de.hsrm.sls.ejunit.mono.filter.FilteringComputer;
import de.hsrm.sls.trex.TREXSerializer;
import de.hsrm.sls.trex.Testsuite;
import de.hsrm.sls.trex.junit.RecordStateRunListener;
import de.hsrm.sls.trex.junit.RunStateChangedListener;
import org.junit.runner.JUnitCore;
import org.junit.runner.notification.Failure;

public class Runner {

    public static void main(String... args) {
        Thread.currentThread().setUncaughtExceptionHandler(new UncaughtExceptionHandler());
        var params = RunArgs.parse(args);

        System.out.println("EJUnit (Mono) started");
        var testsuite = runTests(params);

        System.out.println("Finished test run: " + testsuite.getFailures() + " failures, " +
            testsuite.getErrors() + " errors, " +
            testsuite.getTimeouts() + " timeouts, " +
            //result.getIgnoreCount() + " ignored, " +
            (testsuite.getTestcases().size()) + " total. " +
            "Elapsed: " + testsuite.getTotalTime() + " seconds");
    }

    public static Testsuite runTests(RunArgs args) {
        var filterBuilder = new ExecFilterBuilder();
        var filter = filterBuilder.fromSelectors(args.getSelectors());

        JUnitCore jUnitCore = new JUnitCore();

        RunStateChangedListener listener = null;
        if (args.getOutPath() != null) {
            var trexSerializer = new TREXSerializer(args.getOutPath());
            listener = state -> trexSerializer.serialize(state.toTestsuite());
        }

        var recordStateListener = new RecordStateRunListener(listener, args.getMaxOutputSize());
        jUnitCore.addListener(recordStateListener);

        if (args.isWithJacoco()) {
            jUnitCore.addListener(new JacocoListener());
        }

        var result = jUnitCore.run(new FilteringComputer(filter.identifiers()), filter.testClasses());
        var runnerExceptions = result.getFailures().stream()
            .filter(f -> JUnitUtils.isJUnitErrorTestcase(f.getDescription()))
            .map(Failure::getException)
            .toList();

        if (runnerExceptions.size() != 0) {
            throw new RunListenerException(runnerExceptions);
        }

        return recordStateListener.getRunState().toTestsuite();
    }
}
