package de.hsrm.sls.ejunit.mono.filter;

import de.hsrm.sls.ejunit.common.TestLoader;
import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import java.util.ArrayList;

public class ExecFilterBuilder {

    public ExecFilter fromSelectors(String[] selectors) {
        var testLoader = new TestLoader();

        var identifiers = new ArrayList<TestcaseIdentifier>();
        var classes = new ArrayList<Class>();

        for(var selector : selectors) {
            // test class and method
            if(selector.contains("#")) {
                var identifier = TestcaseIdentifier.fromString(selector);
                var test = testLoader.getTest(identifier.className());
                var methods = test.getMethods();
                if(!methods.contains(identifier.methodName())) {
                    throw new RuntimeException("test method %s does not exist in test class %s".formatted(identifier.methodName(), identifier.className()));
                }

                if(!classes.contains(test.getClazz())) {
                    classes.add(test.getClazz());
                }

                if(!identifiers.contains(identifier)) {
                    identifiers.add(identifier);
                }
            } else {
                // test class only -> add all test methods
                var test = testLoader.getTest(selector);
                var methods = test.getMethods();

                if(!classes.contains(test.getClazz())) {
                    classes.add(test.getClazz());
                }

                var testIdentifiers = methods.stream().map(m -> new TestcaseIdentifier(test.getClazz().getName(), m)).toList();
                for(var identifier : testIdentifiers) {
                    if(!identifiers.contains(identifier)) {
                        identifiers.add(identifier);
                    }
                }
            }
        }

        return new ExecFilter(
            classes.toArray(new Class[classes.size()]),
            identifiers
        );
    }
}
