package de.hsrm.sls.trex.junit;

import de.hsrm.sls.ejunit.common.JUnitUtils;
import de.hsrm.sls.ejunit.common.StackTraceFilter;
import de.hsrm.sls.ejunit.common.StdCapture;
import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import de.hsrm.sls.trex.ErrorResult;
import de.hsrm.sls.trex.FailureResult;
import de.hsrm.sls.trex.PendingResult;
import de.hsrm.sls.trex.Result;
import de.hsrm.sls.trex.StartedResult;
import de.hsrm.sls.trex.SuccessResult;
import de.hsrm.sls.trex.Testcase;
import de.hsrm.sls.trex.Testsuite;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.junit.runner.Description;
import org.junit.runner.notification.Failure;

public class RunState {

    private List<Description> tests = new ArrayList<>();
    private Map<Description, TestcaseState> testStates = new HashMap<>();
    private Map<Description, LocalDateTime> testStarts = new HashMap<>();
    private Map<Description, LocalDateTime> testEnds = new HashMap<>();

    private Map<Description, StdCapture> testOut = new HashMap<>();

    private Map<Description, StdCapture> testErr = new HashMap<>();

    private Map<Description, Failure> testFailures = new HashMap<>();
    private Map<Description, Failure> beforeAllFailures = new HashMap<>();

    private LocalDateTime startedAt;
    private LocalDateTime finishedAt;


    public LocalDateTime getStartedAt() {
        return startedAt;
    }

    public void setStartedAt(LocalDateTime startedAt) {
        this.startedAt = startedAt;
    }

    public LocalDateTime getFinishedAt() {
        return finishedAt;
    }

    public void setFinishedAt(LocalDateTime finishedAt) {
        this.finishedAt = finishedAt;
    }

    public Map<Description, LocalDateTime> getTestStarts() {
        return testStarts;
    }

    public Map<Description, StdCapture> getTestOut() {
        return testOut;
    }

    public Map<Description, StdCapture> getTestErr() {
        return testErr;
    }

    public Map<Description, TestcaseState> getTestStates() {
        return testStates;
    }

    public Map<Description, Failure> getTestFailures() {
        return testFailures;
    }

    public Map<Description, LocalDateTime> getTestEnds() {
        return testEnds;
    }

    public List<Description> getTests() {
        return tests;
    }


    public Testsuite toTestsuite() {
        var testsuite = new Testsuite();
        testsuite.setStartedAt(startedAt);
        testsuite.setFinishedAt(finishedAt);

        var testcases = new ArrayList<Testcase>();
        for (var test : tests) {
            var testcaseState = testStates.get(test);
            var testcaseId = TestcaseIdentifier.fromJUnit(test);
            var testcase = new Testcase(testcaseId.className(), testcaseId.methodName());

            testcase.setStartedAt(testStarts.get(test));
            testcase.setFinishedAt(testEnds.get(test));

            if (testcaseState == TestcaseState.FINISHED) {
                if (testFailures.containsKey(test)) {
                    var failure = testFailures.get(test);
                    var result = createResultForFailure(failure);
                    testcase.setResult(result);
                } else {
                    testcase.setResult(new SuccessResult());
                }

                var outCapture = testOut.get(test);
                var errCapture = testErr.get(test);

                testcase.setStdout(outCapture.toString());
                testcase.setStderr(errCapture.toString());
            } else if (testcaseState == TestcaseState.PENDING) {
                testcase.setResult(new PendingResult());
            } else {
                testcase.setResult(new StartedResult());
            }

            testcases.add(testcase);
        }

        testsuite.setTestcases(testcases);
        return testsuite;
    }

    private Result createResultForFailure(Failure failure) {
        String message = failure.getMessage();

        var stackTraceFilter = new StackTraceFilter();
        String strace = stackTraceFilter.applyFilter(failure.getException());

        if (JUnitUtils.isAssertionFailure(failure)) {
            var result = new FailureResult();
            if (message != null && message.length() > 0) {
                result.setMessage(message);
            }
            result.setDetails(strace);
            return result;
        } else {
            var result = new ErrorResult();
            if (message != null && message.length() > 0) {
                result.setMessage(message);
            }
            result.setDetails(strace);
            return result;
        }
    }
}
