package de.hsrm.sls.sqlunit;

import de.hsrm.sls.sqlunit.evaluation.SQLConfig;
import org.apache.commons.cli.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class RunArgs {
    private String outPath;
    private List<String> solutionFiles;
    private String[] testFiles;
    private SQLConfig sqlConfig;

    public String[] getTestFiles() {
        return testFiles;
    }

    public void setTestFiles(String[] testFiles) {
        this.testFiles = testFiles;
    }

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    public List<String> getSolutionFiles() {
        return solutionFiles;
    }

    public void setSolutionFiles(List<String> solutionFiles) {
        this.solutionFiles = solutionFiles;
    }

    public SQLConfig getSqlConfig() {
        return sqlConfig;
    }

    public void setSqlConfig(SQLConfig sqlConfig) {
        this.sqlConfig = sqlConfig;
    }

    private static final String OPT_OUT_PATH = "out";
    private static final String OPT_SOLUTION_PATH = "solution";

    private static CommandLine parseArgs(String[] args) {
        var options = new Options();

        var solutionPath = new Option("s", OPT_SOLUTION_PATH, true,
                "Comma separated list of paths to the solution files.");
        solutionPath.setRequired(true);
        options.addOption(solutionPath);

        var outPath = new Option("o", OPT_OUT_PATH, true,
                "Output path for the resulting TREX report.");
        outPath.setRequired(false);
        options.addOption(outPath);

        var parser = new DefaultParser();
        var formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp(
                    "sqlunit [OPTIONS] solutionFile [testFile1 [testFile2 [testFile3] ...]]",
                    "Runs all tests in the test files by executing the corresponding SQL queries in the solution files and comparing the result with the expected result..\n\nOPTIONS:",
                    options,
                    "\nIf all tests have finished, exit with status 0. If an unexpected exception occurs in the test runner, return status 1.");
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var trexOutPath = cmd.getOptionValue(OPT_OUT_PATH, "trex_report.xml");
        var solutionFilesRaw = cmd.getOptionValue(OPT_SOLUTION_PATH, "solution.sql");
        var solutionFiles = solutionFilesRaw != null && !solutionFilesRaw.isEmpty()
                ? Arrays.stream(solutionFilesRaw.split(",")).toList()
                : new ArrayList<String>();

        var params = new RunArgs();
        params.setSolutionFiles(solutionFiles);
        params.setOutPath(trexOutPath);
        params.setTestFiles(cmd.getArgs());
        params.setSqlConfig(new SQLConfig(
        "localhost",
        3306,
        "test",
        "root",
        "root"
        ));
        return params;
    }
}
