package de.hsrm.sls.sqlunit;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.sqlunit.evaluation.SimpleQueryEvaluator;
import de.hsrm.sls.sqlunit.segment.Segment;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;

import de.hsrm.sls.sqlunit.segment.SegmentParser;
import de.hsrm.sls.sqlunit.trex.TREXSerializer;
import de.hsrm.sls.sqlunit.trex.Testsuite;

public class Runner {

  public static void main(String[] args) {
    try {
      var params = RunArgs.parse(args);
      System.out.println("SQLUnit started");
      var testsuite = runTests(params);

      System.out.println("Finished test run: " + testsuite.getFailures() + " failures, " +
              testsuite.getErrors() + " errors, " +
              testsuite.getTimeouts() + " timeouts, " +
              //result.getIgnoreCount() + " ignored, " +
              (testsuite.getTestcases().size()) + " total. " +
              "Elapsed: " + testsuite.getTotalTime() + " seconds");
    } catch (Exception e) {
      // Something is wrong with the runner
      e.printStackTrace();
      System.exit(1);
    }
  }

  public static Testsuite runTests(RunArgs args) throws IOException {
    var evaluator = new SimpleQueryEvaluator(args.getSqlConfig());

    var segmentParser = new SegmentParser();

    var solutions = new ArrayList<Segment>();
    for(var solutionFile : args.getSolutionFiles()) {
      var cs = Files.readString(Path.of(solutionFile));
      solutions.addAll(segmentParser.parse(cs));
    }

    var specifications = new ArrayList<Segment>();
    for(var testFile : args.getTestFiles()) {
      var cs = Files.readString(Path.of(testFile));
      specifications.addAll(segmentParser.parse(cs));
    }

    var objectMapper = new ObjectMapper();
    var testrunner = new TestRunner(evaluator, objectMapper);

    var serializer = new TREXSerializer();
    testrunner.setListener(testsuite -> {
      var xml = serializer.serialize(testsuite);
      try {
          Files.writeString(Path.of(args.getOutPath()), xml);
      } catch (IOException e) {
          throw new RuntimeException(e);
      }
    });

    return testrunner.runTests(solutions, specifications);
  }
}
