package de.hsrm.sls.sqlunit.evaluation;

import org.antlr.v4.runtime.BaseErrorListener;
import org.antlr.v4.runtime.RecognitionException;
import org.antlr.v4.runtime.Recognizer;
import org.antlr.v4.runtime.misc.Utils;

import java.util.ArrayList;
import java.util.List;

class SyntaxErrorListener extends BaseErrorListener {
    public class SyntaxError {
        private final Recognizer<?, ?> recognizer;
        private final Object offendingSymbol;
        private final int line;
        private final int charPositionInLine;
        private final String message;
        private final RecognitionException e;

        SyntaxError(
                Recognizer<?, ?> recognizer, Object offendingSymbol, int line,
                int charPositionInLine, String msg, RecognitionException e
        ) {
            this.recognizer = recognizer;
            this.offendingSymbol = offendingSymbol;
            this.line = line;
            this.charPositionInLine = charPositionInLine;
            this.message = msg;
            this.e = e;
        }

        public Recognizer<?, ?> getRecognizer() {
            return recognizer;
        }

        public Object getOffendingSymbol() {
            return offendingSymbol;
        }

        public int getLine() {
            return line;
        }

        public int getCharPositionInLine() {
            return charPositionInLine;
        }

        public String getMessage() {
            return message;
        }

        public RecognitionException getException() {
            return e;
        }
    }

    private final List<SyntaxError> syntaxErrors = new ArrayList<>();

    SyntaxErrorListener() {
    }

    List<SyntaxError> getSyntaxErrors() {
        return syntaxErrors;
    }

    @Override
    public void syntaxError(
            Recognizer<?, ?> recognizer,
            Object offendingSymbol,
            int line, int charPositionInLine,
            String msg, RecognitionException e
    ) {
        syntaxErrors.add(
                new SyntaxError(recognizer, offendingSymbol, line, charPositionInLine, msg, e));
    }

    @Override
    public String toString() {
        return Utils.join(syntaxErrors.iterator(), "\n");
    }
}
