package de.hsrm.sls.sqlunit.trex;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;

public class Testsuite {

    private LocalDateTime startedAt;
    private LocalDateTime finishedAt;
    private List<Testcase> testcases = new ArrayList<>();

    public LocalDateTime getStartedAt() {
        return startedAt;
    }

    public List<Testcase> getTestcases() {
        return testcases;
    }

    public LocalDateTime getFinishedAt() {
        return finishedAt;
    }


    public Duration getDuration() {
        return Duration.between(startedAt.toInstant(ZoneOffset.UTC),
            finishedAt.toInstant(ZoneOffset.UTC));
    }

    public double getTotalTime() {
        final double ONE_SECOND = 1000.0;
        return getDuration().toMillis() / ONE_SECOND;
    }


    public long getTimeouts() {
        return testcases.stream()
            .filter(t -> t.getResult() instanceof TimeoutResult)
            .count();
    }

    public long getFailures() {
        return testcases.stream()
            .filter(t -> t.getResult() instanceof FailureResult)
            .count();
    }

    public long getErrors() {
        return testcases.stream()
            .filter(t -> t.getResult() instanceof ErrorResult)
            .count();
    }

    public void setFinishedAt(LocalDateTime finishedAt) {
        this.finishedAt = finishedAt;
    }

    public void setStartedAt(LocalDateTime startedAt) {
        this.startedAt = startedAt;
    }

    public void setTestcases(List<Testcase> testcases) {
        this.testcases = testcases;
    }
}
