package de.hsrm.sls.sqlunit.segment;


import static org.junit.Assert.assertEquals;

import org.junit.Test;

public class SegmentParserTest {
  @Test
  public void test1(){
    var s = """
        -- QUERY i1:

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(1, segments.size());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;", segments.get(0).content());
  }

  @Test
  public void test2(){
    var s = """
    -- QUERY c1:

    CREATE OR REPLACE TABLE lagerbestand (
    a_nr VARCHAR(13), bestand INT DEFAULT 3,
    PRIMARY KEY (a_nr),
    FOREIGN KEY (a_nr) REFERENCES medienartikel(a_nr)
    ON DELETE CASCADE
    ON UPDATE CASCADE);

    -- QUERY i1:

    INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
    """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(2, segments.size());
    assertEquals("""
    CREATE OR REPLACE TABLE lagerbestand (
    a_nr VARCHAR(13), bestand INT DEFAULT 3,
    PRIMARY KEY (a_nr),
    FOREIGN KEY (a_nr) REFERENCES medienartikel(a_nr)
    ON DELETE CASCADE
    ON UPDATE CASCADE);""", segments.get(0).content());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;", segments.get(1).content());
  }

  @Test
  public void test3(){
    var s = """
        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(0, segments.size());
  }

  @Test
  public void test4(){
    var s = """
        -- QUERY c1

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(1, segments.size());
    assertEquals("c1", segments.get(0).id());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;", segments.get(0).content());
  }

  @Test
  public void test5(){
    var s = """
        -- QUERY:

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(0, segments.size());
  }

  @Test
  public void test6(){
    var s = """
        -- QUERY c1:

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;

        -- QUERY c2

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel2;

        -- QUERY c3:

        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel3;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(3, segments.size());
    assertEquals("c1", segments.get(0).id());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;", segments.get(0).content());
    assertEquals("c2", segments.get(1).id());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel2;", segments.get(1).content());
    assertEquals("c3", segments.get(2).id());
    assertEquals("INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel3;", segments.get(2).content());
  }

  @Test
  public void test7(){
    var s = """
        -- QUERY c1: INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(0, segments.size());
  }

  @Test
  public void test8(){
    var s = """
        -- QUERY c1 INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(0, segments.size());
  }

  @Test
  public void test9(){
    var s = """
        -- QUERY c1:
        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel; -- QUERY c3:
        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel2;
        """;

    var parser = new SegmentParser();
    var segments = parser.parse(s);
    assertEquals(1, segments.size());
    assertEquals("c1", segments.get(0).id());
    assertEquals("""
        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel; -- QUERY c3:
        INSERT INTO lagerbestand(a_nr) SELECT a_nr FROM medienartikel2;""", segments.get(0).content());
  }
}
